/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import java.net.UnknownHostException;
import java.util.HashMap;
import java.util.Map;
import java.util.Set;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.admin.api.PetalsAdministration;
import org.ow2.petals.admin.api.exception.ContainerAdministrationException;
import org.ow2.petals.admin.api.exception.DuplicatedServiceException;
import org.ow2.petals.admin.api.exception.MissingServiceException;
import org.ow2.petals.cli.Constants;
import org.ow2.petals.cli.Main;
import org.ow2.petals.cli.api.command.CommandUtil;
import org.ow2.petals.cli.api.command.ConnectionCommand;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.cli.api.command.exception.ConnectionErrorException;
import org.ow2.petals.cli.api.command.exception.NoDefaultConnectionException;
import org.ow2.petals.cli.api.command.exception.UnexistingAliasException;
import org.ow2.petals.cli.api.connection.AuthenticatedConnectionParameters;
import org.ow2.petals.cli.api.connection.ConnectionParameters;
import org.ow2.petals.cli.api.connection.exception.HostMissingException;
import org.ow2.petals.cli.api.connection.exception.InvalidConnectionParameterException;
import org.ow2.petals.cli.api.connection.exception.InvalidPortException;
import org.ow2.petals.cli.api.connection.exception.PasswordMissingException;
import org.ow2.petals.cli.api.connection.exception.UsernameMissingException;
import org.ow2.petals.cli.api.exception.NoInteractiveShellException;
import org.ow2.petals.cli.api.pref.PreferenceFileException;
import org.ow2.petals.cli.api.pref.Preferences;
import org.ow2.petals.cli.api.shell.Shell;
import org.ow2.petals.cli.api.shell.exception.ShellException;
import org.ow2.petals.cli.connection.AuthenticatedConnectionParametersImpl;

import jline.console.completer.Completer;
import jline.console.completer.StringsCompleter;

/**
 * @author Nicolas Oddoux - EBM WebSourcing
 */
public class Connect extends AbstractPetalsCliCommand implements ConnectionCommand {

    /**
     * The short name of the option setting the alias of the Petals node to connect
     */
    public static final String ALIAS_SHORT_OPTION = "a";

    private static final String ALIAS_LONG_OPTION = "alias";

    private static final String ALIAS_ARG_NAME = "alias";

    private static final Option ALIAS_OPTION = Option.builder(ALIAS_SHORT_OPTION).numberOfArgs(1)
            .argName(ALIAS_ARG_NAME).longOpt(ALIAS_LONG_OPTION).desc("Connection alias in the preference file.")
            .build();

    /**
     * The option setting the host of the Petals node to connect
     */
    public static final String HOST_SHORT_OPTION = "h";

    private static final String HOST_LONG_OPTION = "host";

    private static final String HOST_ARG_NAME = "host";

    private static final Option HOST_OPTION = Option.builder(HOST_SHORT_OPTION).numberOfArgs(1).argName(HOST_ARG_NAME)
            .longOpt(HOST_LONG_OPTION).desc("JMX host of the remote petals ESB.").build();

    /**
     * The option setting the JMX port of the Petals node to connect
     */
    public static final String PORT_SHORT_OPTION = "n";

    private static final String PORT_LONG_OPTION = "port";

    private static final String PORT_ARG_NAME = "port";

    private static final Option PORT_OPTION = Option.builder(PORT_SHORT_OPTION).numberOfArgs(1).argName(PORT_ARG_NAME)
            .longOpt(PORT_LONG_OPTION).desc("JMX port of the remote petals ESB.").build();

    /**
     * The option setting the JMX user of the Petals node to connect
     */
    public static final String USER_SHORT_OPTION = "u";

    private static final String USER_LONG_OPTION = "user";

    private static final String USER_ARG_NAME = "user";

    private static final Option USER_OPTION = Option.builder(USER_SHORT_OPTION).numberOfArgs(1).argName(USER_ARG_NAME)
            .longOpt(USER_LONG_OPTION).desc("JMX user of the remote petals ESB.").build();

    /**
     * The option setting the JMX password of the Petals node to connect
     */
    public static final String PWD_SHORT_OPTION = "p";

    private static final String PWD_LONG_OPTION = "password";

    private static final String PWD_ARG_NAME = "password";

    private static final Option PWD_OPTION = Option.builder(PWD_SHORT_OPTION).numberOfArgs(1).argName(PWD_ARG_NAME)
            .longOpt(PWD_LONG_OPTION).desc("JMX password of the remote petals ESB.").build();

    /**
     * The 'yes' flag option to skip confirmation
     */
    public static final String YESFLAG_SHORT_OPTION = "y";

    private static final Option YESFLAG_OPTION = Option.builder(YESFLAG_SHORT_OPTION).hasArg(false)
            .desc("A flag to skip confirmation.").build();

    public static final String USERNAME_QUESTION = "Username: ";

    public static final String PWD_QUESTION = "Password: ";

    public Connect() {
        super();
        this.setUsage(CommandUtil.formatCommandUsage(this));
        this.setDescription("Connect to a Petals ESB node");
        this.setOptionsDescription(CommandUtil.formatCommandOptionsDescription(this));
    }

    @Override
    public Options createOptions() {
        final Options options = new Options();
        options.addOption(ALIAS_OPTION);
        options.addOption(HOST_OPTION);
        options.addOption(PORT_OPTION);
        options.addOption(USER_OPTION);
        options.addOption(PWD_OPTION);
        options.addOption(YESFLAG_OPTION);
        return options;
    }

    /**
     * {@inheritDoc}
     * 
     * <p>
     * Dedicated processing to the command 'connect': No connection is required
     * to execute it.
     * </p>
     */
    @Override
    public boolean isConnectionRequired() {
        return false;
    }

    @Override
    public void doExecute(final String[] args) throws CommandException {
        final Shell shell = this.getShell();

        try {
            final CommandLineParser parser = new DefaultParser();
            final CommandLine cmd = parser.parse(this.getOptions(), args);

            final ConnectionParameters connectionParameters = this
                    .parseConnectionParameters(cmd, true);

            if (cmd.getOptions().length == 0) {
                final String confirmationQuestion;
                final String host = connectionParameters.getHost();
                final int port = connectionParameters.getPort();
                if (connectionParameters instanceof AuthenticatedConnectionParametersImpl) {
                    final String username = ((AuthenticatedConnectionParametersImpl) connectionParameters)
                            .getUsername();
                    confirmationQuestion = String.format(
                            "Would you like to connect to %s:*****@%s:%d? (y/n) ", username, host, port);
                } else {
                    confirmationQuestion = String.format(
                            "Would you like to connect to %s:%d? (y/n) ", host, port);
                }
                if (cmd.hasOption(YESFLAG_SHORT_OPTION) || shell.confirms(confirmationQuestion)) {
                    this.getShell().setConnectionParameters(
                            this.connect(connectionParameters, cmd.hasOption(YESFLAG_SHORT_OPTION)));
                }

            } else {
                final AuthenticatedConnectionParameters authenticatedConnectionParameters = this
                        .connect(connectionParameters, cmd.hasOption(YESFLAG_SHORT_OPTION));
                this.getShell().setConnectionParameters(authenticatedConnectionParameters);
                if(authenticatedConnectionParameters != null) {
                    final String confirmation = String.format("Connected on %s:%d with '%s'",
                            authenticatedConnectionParameters.getHost(), authenticatedConnectionParameters.getPort(),
                            ((AuthenticatedConnectionParametersImpl) authenticatedConnectionParameters).getUsername());
                    shell.getPrintStream().println(confirmation);
                }
            }
        } catch (final UnrecognizedOptionException e) {
            throw new CommandBadArgumentNumberException(this, e);
        } catch (final MissingArgumentException e) {
            throw new CommandMissingArgumentException(this, e.getOption(), e);
        } catch (final MissingOptionException e) {
            throw new CommandMissingOptionsException(this, e.getMissingOptions(), e);
        } catch (final ParseException e) {
            throw new CommandInvalidException(this, e);
        } catch (final PreferenceFileException e) {
            throw new CommandException(this, e);
        } catch (final ShellException e) {
            throw new CommandException(this, e);
        } catch (final InvalidConnectionParameterException e) {
            throw new CommandException(this, e);
        }
    }

    @Override
    public Map<String, Completer> getOptionCompleters() {
        try {
            return this.getConnectionOptionCompleters();
        } catch (final PreferenceFileException pfe) {
            // no available completion
            return new HashMap<String, Completer>();
        }
    }

    @Override
    public AuthenticatedConnectionParameters connect(final ConnectionParameters connectionParameters,
            final boolean isYesFlagEnabledAtCmdLevel) throws ConnectionErrorException, CommandException {

        final String host = connectionParameters.getHost();
        final int port = connectionParameters.getPort();
        AuthenticatedConnectionParametersImpl authenticatedConnectionParameters;
        if (connectionParameters instanceof AuthenticatedConnectionParametersImpl) {
            authenticatedConnectionParameters = (AuthenticatedConnectionParametersImpl) connectionParameters;
            connect(host, port, authenticatedConnectionParameters.getUsername(),
                    authenticatedConnectionParameters.getPassword(), this.getShell());
        } else if (!isYesFlagEnabledAtCmdLevel) {
            try {
                do {
                    final String username = this.getShell().askQuestion(USERNAME_QUESTION, false);
                    final String password = this.getShell().askQuestion(PWD_QUESTION, true);
                    try {
                        connect(host, port, username, password, this.getShell());
                        authenticatedConnectionParameters = new AuthenticatedConnectionParametersImpl(host, port,
                                username,
                                password, null);
                        break;
                    } catch (final UnknownHostException e) {
                        throw new CommandInvalidArgumentException(null, HOST_OPTION, host, e);
                    } catch (final HostMissingException e) {
                        throw new CommandInvalidArgumentException(null, HOST_OPTION, host, e);
                    } catch (final InvalidPortException e) {
                        throw new CommandInvalidArgumentException(null, PORT_OPTION, host, e);
                    } catch (final UsernameMissingException e) {
                        throw new CommandInvalidArgumentException(null, USER_OPTION, host, e);
                    } catch (final PasswordMissingException e) {
                        throw new CommandInvalidArgumentException(null, PWD_OPTION, host, e);
                    } catch (final ConnectionErrorException cae) {
                        if (cae.getMessage().endsWith("Authentication failed! Invalid username or password")) {
                            if (!this.getShell().confirms("Retry? (y/n) ")) {
                                authenticatedConnectionParameters = null;
                                break;
                            }
                        } else {
                            throw new ConnectionErrorException(this, cae);
                        }
                    }
                } while (true);
            } catch (final NoInteractiveShellException e) {
                throw new CommandException(null, Constants.CommandMessages.MISSING_USERNAME_AND_PASSWORD, e);
            } catch (final ShellException e) {
                throw new CommandException(null, e);
            }
        } else {
            // 'yes' flag enabled at command level
            throw new CommandException(null, Constants.CommandMessages.MISSING_USERNAME_AND_PASSWORD);
        }

        return authenticatedConnectionParameters;
    }

    /**
     * @param host
     * @param port
     * @param username
     * @param password
     * @param shell
     * @throws ConnectionErrorException
     *             An error occurs establishing the connection
     */
    private void connect(final String host, final int port, final String username, final String password,
            final Shell shell) throws ConnectionErrorException {

        try {
            final PetalsAdministration containerAdministration = getAdmin();
            containerAdministration.connect(host, port, username, password);
            shell.setPrompt(String.format("%s@%s:%d>", Main.PROMPT_BASE, host, port));
            shell.onConnectionEstablished();
        } catch (final DuplicatedServiceException e) {
            throw new ConnectionErrorException(this, e);
        } catch (final MissingServiceException e) {
            throw new ConnectionErrorException(this, e);
        } catch (final ContainerAdministrationException e) {
            throw new ConnectionErrorException(this, e);
        }
    }

    @Override
    public ConnectionParameters parseConnectionParameters(final CommandLine cmd, final boolean useDefaultConnection)
            throws CommandException, PreferenceFileException, InvalidConnectionParameterException {
        final Preferences pref = this.getShell().getPreferences();
        if (pref != null) {
            final Map<String, ConnectionParameters> preferenceConnectionParameters = this.getShell().getPreferences()
                    .getPreferenceConnectionParameters();
            final ConnectionParameters defaultConnectionParameters = this.getShell().getPreferences()
                    .getDefaultConnectionParameters();
            return parseConnectionParameters(cmd, useDefaultConnection, preferenceConnectionParameters,
                    defaultConnectionParameters);
        } else {
            return parseConnectionParameters(cmd, false, null, null);
        }
    }

    private Map<String, Completer> getConnectionOptionCompleters() throws PreferenceFileException {
        final Map<String, ConnectionParameters> preferenceConnectionParameters = this.getShell().getPreferences()
                .getPreferenceConnectionParameters();
        final Set<String> aliasSet = preferenceConnectionParameters.keySet();
        final Map<String, Completer> optionCompleters = new HashMap<String, Completer>();
        optionCompleters.put(ALIAS_SHORT_OPTION, new StringsCompleter(aliasSet.toArray(new String[aliasSet.size()])));
        return optionCompleters;
    }

    private ConnectionParameters parseConnectionParameters(final CommandLine cmd, final boolean useDefaultConnection,
            final Map<String, ConnectionParameters> preferenceConnectionParameters,
            final ConnectionParameters defaultConnectionParameters) throws CommandException, PreferenceFileException,
            InvalidConnectionParameterException {
        ConnectionParameters connectionParameters;

        if (cmd.hasOption(HOST_SHORT_OPTION) && cmd.hasOption(PORT_SHORT_OPTION) && cmd.hasOption(USER_SHORT_OPTION)
                && cmd.hasOption(PWD_SHORT_OPTION) && !cmd.hasOption(ALIAS_SHORT_OPTION)) {
            final String host = cmd.getOptionValue(HOST_SHORT_OPTION);
            final String jmxPortStr = cmd.getOptionValue(PORT_SHORT_OPTION);
            final String username = cmd.getOptionValue(USER_SHORT_OPTION);
            final String password = cmd.getOptionValue(PWD_SHORT_OPTION);
            try {
                final int port = Integer.parseInt(jmxPortStr);
                connectionParameters = new AuthenticatedConnectionParametersImpl(host, port, username, password, null);
            } catch (final NumberFormatException nfe) {
                throw new CommandInvalidArgumentException(this, PORT_OPTION, jmxPortStr, nfe);
            } catch (final UnknownHostException e) {
                throw new CommandInvalidArgumentException(this, HOST_OPTION, host, e);
            }
        } else if (!cmd.hasOption(HOST_SHORT_OPTION) && !cmd.hasOption(PORT_SHORT_OPTION)
                && !cmd.hasOption(USER_SHORT_OPTION) && !cmd.hasOption(PWD_SHORT_OPTION)
                && cmd.hasOption(ALIAS_SHORT_OPTION)) {
            connectionParameters = preferenceConnectionParameters.get(cmd.getOptionValue(ALIAS_SHORT_OPTION));
            if (connectionParameters == null) {
                throw new UnexistingAliasException(this, cmd.getOptionValue(ALIAS_SHORT_OPTION));
            }
        } else if (!cmd.hasOption(HOST_SHORT_OPTION) && !cmd.hasOption(PORT_SHORT_OPTION)
                && !cmd.hasOption(USER_SHORT_OPTION) && !cmd.hasOption(PWD_SHORT_OPTION)
                && !cmd.hasOption(ALIAS_SHORT_OPTION)) {
            if (useDefaultConnection) {
                connectionParameters = defaultConnectionParameters;
                if (connectionParameters == null) {
                    throw new NoDefaultConnectionException(this);
                }
            } else {
                connectionParameters = null;
            }
        } else {
            throw new CommandBadArgumentNumberException(this);
        }

        return connectionParameters;
    }
}
