/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import java.io.PrintStream;
import java.util.List;
import java.util.Map;
import java.util.Map.Entry;
import java.util.Properties;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.apache.commons.cli.MissingArgumentException;
import org.apache.commons.cli.MissingOptionException;
import org.apache.commons.cli.Option;
import org.apache.commons.cli.Options;
import org.apache.commons.cli.ParseException;
import org.apache.commons.cli.UnrecognizedOptionException;
import org.ow2.petals.admin.api.ContainerAdministration;
import org.ow2.petals.admin.api.artifact.Component;
import org.ow2.petals.admin.api.artifact.ServiceAssembly;
import org.ow2.petals.admin.api.artifact.SharedLibrary;
import org.ow2.petals.admin.api.exception.ContainerAdministrationException;
import org.ow2.petals.admin.topology.Container;
import org.ow2.petals.admin.topology.Container.PortType;
import org.ow2.petals.admin.topology.Domain;
import org.ow2.petals.cli.api.command.CommandUtil;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.cli.api.connection.ConnectionParameters;
import org.ow2.petals.cli.connection.AuthenticatedConnectionParametersImpl;
import org.ow2.petals.cli.shell.completer.ContainerNameCompleter;

import jline.console.completer.Completer;

/**
 * @author Nicolas Oddoux - EBM WebSourcing
 */
public class TopologyList extends AbstractPetalsCliCommand {

    private static final String DEFAULT_REGEX_OPTION_VALUE = ".*";

    public static final String PASSPHRASE_SHORT_OPTION = "p";

    public static final String PASSPHRASE_LONG_OPTION = "pass-phrase";

    protected static final Option PASSPHRASE_OPTION = Option.builder(PASSPHRASE_SHORT_OPTION).required(false)
            .numberOfArgs(1).argName("pass-phrase").longOpt(PASSPHRASE_LONG_OPTION)
            .desc("The security pass-phrase to get critical information.").build();

    public static final String CONTAINER_FILTER_SHORT_OPTION = "c";

    public static final String CONTAINER_FILTER_LONG_OPTION = "container";

    protected static final Option CONTAINER_FILTER_OPTION = Option.builder(CONTAINER_FILTER_SHORT_OPTION)
            .required(false).numberOfArgs(1).optionalArg(true).argName("container-name-regexp")
            .longOpt(CONTAINER_FILTER_LONG_OPTION).desc("Regexp to use as container name filter (optional).").build();

    public static final String ARTIFACTS_SHORT_OPTION = "a";

    public static final String ARTIFACTS_LONG_OPTION = "artifacts";

    protected static final Option ARTIFACTS_OPTION = Option.builder(ARTIFACTS_SHORT_OPTION).required(false)
            .hasArg(false).longOpt(ARTIFACTS_LONG_OPTION)
            .desc("JBI artifacts deployed on each container of the topology are displayed.").build();

    public TopologyList() {
        super("topology-list");
        this.setUsage(CommandUtil.formatCommandUsage(this));
        this.setDescription("Get information about the topology this node is part of.");
        this.setOptionsDescription(CommandUtil.formatCommandOptionsDescription(this));
    }

    @Override
    public void doExecute(final String[] args) throws CommandException {

        final CommandLineParser parser = new DefaultParser();
        try {
            final CommandLine cli = parser.parse(this.getOptions(), args);

            final String passPhrase;
            if (cli.hasOption(PASSPHRASE_SHORT_OPTION)) {
                // The pass phrase is provided as argument option
                passPhrase = cli.getOptionValue(PASSPHRASE_SHORT_OPTION);
            } else if (!cli.getArgList().isEmpty()) {
                throw new CommandBadArgumentNumberException(this);
            } else {
                // We check that the pass phrase is set at alias level
                final ConnectionParameters connectionParameters = this.getShell().getConnectionParameters();
                if (connectionParameters != null
                        && connectionParameters instanceof AuthenticatedConnectionParametersImpl) {
                    passPhrase = ((AuthenticatedConnectionParametersImpl) connectionParameters).getPassPhrase();
                } else {
                    passPhrase = null;
                }
            }

            final boolean displaysJbiArtifacts = cli.hasOption(ARTIFACTS_SHORT_OPTION);
            if (displaysJbiArtifacts && passPhrase == null) {
                throw new CommandBadArgumentNumberException(this);
            }

            final ContainerAdministration containerAdministration = getAdmin().newContainerAdministration();
            final Properties currentServerProps = containerAdministration.getServerProperties();
            // TODO: The property 'petals.container.name' must be defined in an API usable by Petals CLI
            final String defaultContainerRegexp = currentServerProps == null ? null
                    : currentServerProps.getProperty("petals.container.name");

            final String containerNameRegex = getRegexOption(cli, CONTAINER_FILTER_SHORT_OPTION,
                    defaultContainerRegexp);

            final Domain domain = containerAdministration.getTopology(containerNameRegex, passPhrase,
                    displaysJbiArtifacts);
            if (domain == null) {
                throw new CommandException(this, "No Petals domain found");
            }

            final PrintStream out = this.getShell().getPrintStream();
            out.println(buildOutput(domain, displaysJbiArtifacts).toString());

        } catch (final UnrecognizedOptionException e) {
            throw new CommandBadArgumentNumberException(this, e);
        } catch (final MissingArgumentException e) {
            throw new CommandMissingArgumentException(this, e.getOption(), e);
        } catch (final MissingOptionException e) {
            throw new CommandMissingOptionsException(this, e.getMissingOptions(), e);
        } catch (final ParseException e) {
            throw new CommandInvalidException(this, e);
        } catch (final ContainerAdministrationException e) {
            throw new CommandException(this, e);
        }
    }

    /**
     * Build the command output.
     * 
     * @param domain
     * @param displaysJbiArtifacts
     * @return
     */
    private static StringBuilder buildOutput(final Domain domain, final boolean displaysJbiArtifacts) {
        final StringBuilder output = new StringBuilder(String.format("Domain: %s%n", domain.getName()));

        for (final Container container : domain.getContainers()) {
            addContainerInformation(container, displaysJbiArtifacts, output);
        }
        return output;
    }

    /**
     * Add container information to the command output
     * 
     * @param container
     * @param displaysJbiArtifacts
     * @param output
     */
    private static void addContainerInformation(final Container container, final boolean displaysJbiArtifacts,
            final StringBuilder output) {
        final String containerInformation = String.format("* Container: %s (%s)%n** Host: %s%n",
                container.getContainerName(), container.getState(), container.getHost());
        output.append(containerInformation);

        final Map<PortType, Integer> ports = container.getPorts();
        for (final Entry<PortType, Integer> entry : ports.entrySet()) {
            output.append(String.format("** %s: %d%n", entry.getKey(), entry.getValue()));
        }

        final String jmxUser = container.getJmxUsername();
        output.append(
                String.format("** JMX username: %s%n", (jmxUser == null || jmxUser.isEmpty()) ? "********" : jmxUser));
        final String jmxPassword = container.getJmxPassword();
        output.append(String.format("** JMX password: %s%n",
                (jmxPassword == null || jmxPassword.isEmpty()) ? "********" : jmxPassword));

        if (displaysJbiArtifacts) {
            addArtifacts(container, output);
        }
    }

    /**
     * Add container artifacts information to the command output
     * 
     * @param container
     * @param output
     */
    private static void addArtifacts(final Container container, final StringBuilder output) {
        // JBI Artifacts displaying is requested
        final List<SharedLibrary> sharedLibraries = container.getSharedLibraries();
        if (!sharedLibraries.isEmpty()) {
            output.append("** Shared libraries:\n");
            for (final SharedLibrary sharedLibrary : sharedLibraries) {
                output.append(String.format("*** %s (%s)%n", sharedLibrary.getName(), sharedLibrary.getVersion()));
            }
        }

        final List<Component> components = container.getComponents();
        if (!components.isEmpty()) {
            output.append("** Components:\n");
            for (final Component component : components) {
                output.append(String.format("*** %s (%s)%n", component.getName(), component.getState()));
            }
        }

        final List<ServiceAssembly> serviceAssemblies = container.getServiceAssemblies();
        if (!serviceAssemblies.isEmpty()) {
            output.append("** Service assemblies:\n");
            for (final ServiceAssembly serviceAssembly : serviceAssemblies) {
                output.append(String.format("*** %s (%s)%n", serviceAssembly.getName(), serviceAssembly.getState()));
            }
        }
    }

    @Override
    public Options createOptions() {
        final Options options = new Options();
        options.addOption(CONTAINER_FILTER_OPTION);
        options.addOption(PASSPHRASE_OPTION);
        options.addOption(ARTIFACTS_OPTION);
        return options;
    }

    @Override
    public Map<String, Completer> getOptionCompleters() {
        this.completers.put(CONTAINER_FILTER_SHORT_OPTION,
                new ContainerNameCompleter(getAdmin().newContainerAdministration()));
        return this.completers;
    }

    /**
     * 
     * @param cmd
     * @param option
     * @param defaultRegexp
     *            A default regexp to use if no option value is defined
     * @return
     */
    private static final String getRegexOption(final CommandLine cmd, final String option, final String defaultRegexp) {
        String regexp;
        if (cmd.hasOption(option)) {
            regexp = cmd.getOptionValue(option);
        } else {
            // if the option is not present, we match on all!
            regexp = DEFAULT_REGEX_OPTION_VALUE;
        }
        if (regexp == null) {
            regexp = defaultRegexp == null ? DEFAULT_REGEX_OPTION_VALUE : defaultRegexp;
        }
        return regexp;
    }
}
