/**
 * Copyright (c) 2014-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.util;

import java.net.InetAddress;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.net.URL;
import java.net.UnknownHostException;
import java.util.logging.Logger;

import org.ow2.petals.admin.api.exception.ArtifactUrlRewriterException;
import org.ow2.petals.cli.api.exception.EmbeddedHttpServerException;
import org.ow2.petals.cli.api.http.EmbeddedHttpServer;
import org.ow2.petals.cli.api.shell.Shell;
import org.ow2.petals.cli.api.shell.ShellExtension;
import org.ow2.petals.cli.shell.extension.HttpServerShellExtension;

/**
 * An artifact URL rewriter transforming local URL into an URL of the HTTP server embedded by Petals CLI
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ArtifactUrlRewriter implements org.ow2.petals.admin.api.conf.ArtifactUrlRewriter {

    private final Logger log = Logger.getLogger(ArtifactUrlRewriter.class.getName());

    /**
     * The current Petals CLI shell
     */
    private final Shell shell;

    public ArtifactUrlRewriter(final Shell shell) {
        this.shell = shell;
    }

    @Override
    public URL rewrite(final URL artifactUrl) throws ArtifactUrlRewriterException {

        if (artifactUrl.getProtocol().equals("file")) {
            this.log.fine("The URL identified a local artifact");
            try {
                final String host = this.shell.getConnectionParameters().getHost();
                if (NetworkInterface.getByInetAddress(InetAddress.getByName(host)) != null) {
                    this.log.fine("Connected to a local container");
                    return artifactUrl;
                } else {
                    this.log.fine("Connected to a remote container");
                    // We retrieve the shell extension associated to the embedded HTTP server
                    final ShellExtension[] shellExtensions = this.shell.getExtensions();
                    if (shellExtensions != null) {
                        EmbeddedHttpServer embeddedHttpServer = null;
                        for (final ShellExtension shellExtension : shellExtensions) {
                            if (shellExtension instanceof HttpServerShellExtension) {
                                embeddedHttpServer = ((HttpServerShellExtension) shellExtension)
                                        .getEmbeddedHttpServer();
                                break;
                            }
                        }
                        if (embeddedHttpServer == null) {
                            // It's a bug
                            throw new ArtifactUrlRewriterException(
                                    "BUG: Unable to find the shell extension associated to the embedded HTTP server !");
                        } else {
                            try {
                                return embeddedHttpServer.registerArtifactUrl(artifactUrl);
                            } catch (final EmbeddedHttpServerException e) {
                                throw new ArtifactUrlRewriterException(
                                        "An error occurs starting the embedded HTTP server", e);
                            }
                        }
                    } else {
                        // It's a bug
                        throw new ArtifactUrlRewriterException(
                                "BUG: Unable to find the shell extension associated to the embedded HTTP server because no shell extension are registered !");
                    }
                }
            } catch (final SocketException e) {
                throw new ArtifactUrlRewriterException(
                        "An error occurs determining if the embedded HTTP server is required", e);
            } catch (final UnknownHostException e) {
                throw new ArtifactUrlRewriterException(
                        "An error occurs determining if the embedded HTTP server is required", e);
            }
        } else {
            return artifactUrl;
        }
    }

}
