/**
 * Copyright (c) 2015-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli;

import static org.junit.Assert.assertEquals;

import java.util.Collections;
import java.util.HashMap;
import java.util.Map;
import java.util.Properties;

import org.junit.Before;
import org.junit.Rule;
import org.ow2.petals.admin.api.PetalsAdministration;
import org.ow2.petals.admin.api.exception.ContainerAdministrationException;
import org.ow2.petals.admin.junit.PetalsAdministrationApi;
import org.ow2.petals.admin.junit.utils.SystemInfoUtils;
import org.ow2.petals.admin.topology.Container;
import org.ow2.petals.admin.topology.Container.PortType;
import org.ow2.petals.admin.topology.Container.State;
import org.ow2.petals.cli.api.connection.ConnectionParameters;
import org.ow2.petals.cli.api.pref.PreferenceFileException;
import org.ow2.petals.cli.api.pref.Preferences;
import org.ow2.petals.cli.api.pref.exception.MissingDefaultPreferenceFileException;
import org.ow2.petals.cli.api.pref.exception.PreferenceFileNotFoundException;
import org.ow2.petals.cli.api.shell.ShellExtension;
import org.ow2.petals.cli.base.junit.AbstractConsoleInOutTest;
import org.ow2.petals.cli.connection.AuthenticatedConnectionParametersImpl;
import org.ow2.petals.cli.pref.PreferencesImpl;
import org.ow2.petals.cli.shell.extension.PetalsAdminShellExtension;

/**
 * 
 * @author vnoel
 *
 */
public abstract class AbstractCliTest extends AbstractConsoleInOutTest {

    protected Container container;

    @Rule
    public final PetalsAdministrationApi api = new PetalsAdministrationApi();

    @Before
    public void setUp() {
        api.registerDomain();
        api.registerSystemInfo(SystemInfoUtils.SYSTEM_INFO_EXAMPLE);
        container = new Container("petals", "localhost", Collections.singletonMap(PortType.JMX, 7700), "petals",
                "petals", State.REACHABLE);
        api.registerContainer(container);
    }

    protected void connectSingleton() throws ContainerAdministrationException {
        api.getSingleton().connect(container.getHost(), container.getPorts().get(PortType.JMX),
                container.getJmxUsername(), container.getJmxPassword());
    }

    /**
     * The env variable containing the preference file
     */
    public final static String PREF_ENV_VAR = "PETALS_CLI_PREFS";

    protected void setUpInvalidPrefFile() throws Exception {
        setUpInvalidPrefFile(PREF_ENV_VAR);
    }

    protected void setUpPrefFile(final Properties preferenceProperties) throws Exception {
        setUpPrefFile(PREF_ENV_VAR, preferenceProperties);
    }

    protected void setUpEmptyPrefFile() throws Exception {
        setUpEmptyPrefFile(PREF_ENV_VAR);
    }

    protected PetalsAdminShellExtension runCli(final String... args) {
        return runCli(new PreferencesImpl(), 0, args);
    }

    protected PetalsAdminShellExtension runCli(final Preferences prefs, final String... args) {
        return runCli(prefs, 0, args);
    }

    protected PetalsAdminShellExtension runCli(long expectedExitCode, final String... args) {
        return runCli(new PreferencesImpl(), expectedExitCode, args);
    }

    protected PetalsAdminShellExtension runCli(final Preferences prefs, final long expectedExitCode,
            final String... args) {

        final PetalsAdminShellExtension petalsAdminShellExtension = new PetalsAdminShellExtension();
        
        this.runCli(new Runnable() {

            @Override
            public void run() {

                final int exitCode = new Main().run(args, prefs, new ShellExtension[] { petalsAdminShellExtension });

                assertEquals("invalid exit code", expectedExitCode, exitCode);
            }
        });
        
        return petalsAdminShellExtension;
    }

    protected PetalsAdministration runCliBlocking(final String... args) {
        return runCliBlocking(new PreferencesImpl(), args);
    }

    protected PetalsAdministration runCliBlocking(final Preferences prefs, final String... args) {
        return runCliBlocking(prefs, 0, 0, args);
    }

    protected PetalsAdministration runCliBlocking(final long expectedExitCode, final String... args) {
        return runCliBlocking(new PreferencesImpl(), expectedExitCode, 0, args);
    }

    protected PetalsAdministration runCliBlocking(final long expectedExitCode, final int waitingTime,
            final String... args) {
        return runCliBlocking(new PreferencesImpl(), expectedExitCode, waitingTime, args);
    }

    /**
     * 
     * @param prefs
     *            Preferences
     * @param expectedExitCode
     *            Expected exit code
     * @param waitingTime
     *            Waiting time, in seconds, before to declare test as failed. If {@code 0}, a default value is used.
     * @param args
     *            CLI arguments
     * @return
     */
    protected PetalsAdministration runCliBlocking(final Preferences prefs, final long expectedExitCode,
            final int waitingTime,
            final String... args) {
        PetalsAdminShellExtension admin = runCli(prefs, expectedExitCode, args);
        if (waitingTime == 0) {
            waitThreadOrFail();
        } else {
            waitThreadOrFail(waitingTime * 1000);
        }
        return admin.getAdmin();
    }

    /**
     * Create a basic preference configuration
     * 
     * @param host
     * @param port
     * @param username
     * @param password
     * @param passPhrase
     * @return A basic preference configuration
     */
    protected Preferences createBasicPreferences(final String host, final int port, final String username,
            final String password, final String passPhrase) {

        try {
            final ConnectionParameters connectionParameters = new AuthenticatedConnectionParametersImpl(host, port,
                    username, password, passPhrase);
            return new Preferences() {

                @Override
                public void intializePreferenceParameters() throws MissingDefaultPreferenceFileException,
                        PreferenceFileNotFoundException, PreferenceFileException {
                    // NOP
                }

                @Override
                public Map<String, ConnectionParameters> getPreferenceConnectionParameters() {
                    final Map<String, ConnectionParameters> connectionParametersMap = new HashMap<>();
                    connectionParametersMap.put("default", connectionParameters);
                    return connectionParametersMap;
                }

                @Override
                public ConnectionParameters getDefaultConnectionParameters() {
                    return connectionParameters;
                }
            };
        } catch (final Exception e) {
            // this should never happen
            throw new AssertionError(e);
        }
    }
}
