/**
 * Copyright (c) 2014-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.junit;

import java.io.File;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.net.MalformedURLException;
import java.net.URL;
import java.util.Properties;

import org.junit.rules.ExternalResource;
import org.junit.rules.TemporaryFolder;
import org.ow2.petals.cli.junit.exception.AliasPropertyAlreadySetException;
import org.ow2.petals.cli.junit.exception.DefaultAliasAlreadyExistsException;
import org.ow2.petals.cli.pref.AbstractPreferencesImpl;
import org.ow2.petals.cli.pref.PreferencesImpl;

public class TemporaryPreferencesFile extends ExternalResource {

    /**
     * A classloader reading any file as preference file
     */
    public class PrefFileClassLoader extends ClassLoader {

        private final File prefFile;

        public PrefFileClassLoader(final File prefFile) {
            this.prefFile = prefFile;
        }

        @Override
        public URL getResource(final String name) {
            if (PreferencesImpl.PREFERENCE_FILE_NAME.equals(name)) {
                try {
                    return this.prefFile.toURI().toURL();
                } catch (final MalformedURLException e) {
                    // Should not occur
                    return null;
                }
            } else {
                return super.getResource(name);
            }
        }
    }

    /**
     * A {@link TemporaryFolder} where the perference file will be take place
     */
    private final TemporaryFolder tempFolder = new TemporaryFolder();

    /**
     * The initial classloader before to insert into it the preferences file
     */
    private ClassLoader oldClassLoader = null;

    /**
     * The preference file.
     */
    private File prefFile = null;

    @Override
    protected void before() throws Throwable {
        this.createPrefFile();
        this.setExecutionEnv();
    }

    @Override
    protected void after() {
        try {
            this.deletePrefFile();
        } finally {
            this.restoreExecutionEnv();
        }
    }

    private void createPrefFile() throws IOException {
        this.tempFolder.create();
        this.prefFile = this.tempFolder.newFile();
    }

    private void setExecutionEnv() {
        this.oldClassLoader = Thread.currentThread().getContextClassLoader();
        Thread.currentThread().setContextClassLoader(new PrefFileClassLoader(this.prefFile));
    }

    private void deletePrefFile() {
        this.tempFolder.delete();
    }

    private void restoreExecutionEnv() {
        Thread.currentThread().setContextClassLoader(this.oldClassLoader);
    }

    /**
     * Add an alias to the preferences file
     * 
     * @param alias
     *            Alias to add
     * @throws AliasPropertyAlreadySetException
     *             The property of the alias was already set previously
     * @throws DefaultAliasAlreadyExistsException
     *             The default alias was already set previously
     * @throws IOException
     *             An error occur reading or writing the temporary preferences file.
     */
    public TemporaryPreferencesFile addAlias(final Alias alias)
            throws AliasPropertyAlreadySetException, DefaultAliasAlreadyExistsException, IOException {

        final Properties preferences = new Properties();
        try (final FileInputStream fis = new FileInputStream(this.prefFile)) {
            preferences.load(fis);

            if (alias.isDefaultAlias()) {
                if (preferences.containsKey(AbstractPreferencesImpl.DEFAULT_ALIAS_PROPERTY)) {
                    throw new DefaultAliasAlreadyExistsException();
                } else {
                    preferences.setProperty(AbstractPreferencesImpl.DEFAULT_ALIAS_PROPERTY, alias.getAliasName());
                }
            }

            final String aliasName = alias.getAliasName();
            if (preferences.containsKey(aliasName + AbstractPreferencesImpl.HOST_PROPERTY_SUFFIX)) {
                throw new AliasPropertyAlreadySetException(aliasName, AbstractPreferencesImpl.HOST_PROPERTY_SUFFIX);
            } else if (alias.getHostName() != null) {
                preferences.setProperty(aliasName + AbstractPreferencesImpl.HOST_PROPERTY_SUFFIX, alias.getHostName());
            }

            if (preferences.containsKey(aliasName + AbstractPreferencesImpl.PORT_PROPERTY_SUFFIX)) {
                throw new AliasPropertyAlreadySetException(aliasName, AbstractPreferencesImpl.PORT_PROPERTY_SUFFIX);
            } else if (alias.getPort() != null) {
                preferences.setProperty(aliasName + AbstractPreferencesImpl.PORT_PROPERTY_SUFFIX, alias.getPort());
            }

            if (preferences.containsKey(aliasName + PreferencesImpl.USERNAME_PROPERTY_SUFFIX)) {
                throw new AliasPropertyAlreadySetException(aliasName, PreferencesImpl.USERNAME_PROPERTY_SUFFIX);
            } else if (alias.getJmxUser() != null) {
                preferences.setProperty(aliasName + PreferencesImpl.USERNAME_PROPERTY_SUFFIX, alias.getJmxUser());
            }

            if (preferences.containsKey(aliasName + PreferencesImpl.PASSWORD_PROPERTY_SUFFIX)) {
                throw new AliasPropertyAlreadySetException(aliasName, PreferencesImpl.PASSWORD_PROPERTY_SUFFIX);
            } else if (alias.getJmxPwd() != null) {
                preferences.setProperty(aliasName + PreferencesImpl.PASSWORD_PROPERTY_SUFFIX, alias.getJmxPwd());
            }

            if (preferences.containsKey(aliasName + PreferencesImpl.PASSPHRASE_PROPERTY_SUFFIX)) {
                throw new AliasPropertyAlreadySetException(aliasName, PreferencesImpl.PASSPHRASE_PROPERTY_SUFFIX);
            } else if (alias.getPassPhrase() != null) {
                preferences.setProperty(aliasName + PreferencesImpl.PASSPHRASE_PROPERTY_SUFFIX, alias.getPassPhrase());
            }

            try (final FileOutputStream fos = new FileOutputStream(this.prefFile)) {
                preferences.store(fos, "");
            }
        }

        return this;
    }

}
