/**
 * Copyright (c) 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import static org.hamcrest.Matchers.containsString;
import static org.hamcrest.Matchers.endsWith;
import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.cli.base.junit.Assert.assertCommandInvalidArgumentException;

import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.Writer;

import org.apache.commons.cli.CommandLine;
import org.apache.commons.cli.CommandLineParser;
import org.apache.commons.cli.DefaultParser;
import org.junit.Rule;
import org.junit.Test;
import org.ow2.petals.cli.Constants;
import org.ow2.petals.cli.Main;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandInvalidArgumentException;
import org.ow2.petals.cli.api.connection.ConnectionParameters;
import org.ow2.petals.cli.api.pref.Preferences;
import org.ow2.petals.cli.api.shell.ShellExtension;
import org.ow2.petals.cli.base.junit.shell.DummyShell;
import org.ow2.petals.cli.base.junit.shell.PetalsInteractiveCliUtils;
import org.ow2.petals.cli.connection.AuthenticatedConnectionParametersImpl;
import org.ow2.petals.cli.junit.Alias;
import org.ow2.petals.cli.junit.DefaultAlias;
import org.ow2.petals.cli.junit.TemporaryPreferencesFile;
import org.ow2.petals.cli.junit.exception.AliasPropertyAlreadySetException;
import org.ow2.petals.cli.junit.exception.DefaultAliasAlreadyExistsException;
import org.ow2.petals.cli.pref.PreferencesImpl;
import org.ow2.petals.cli.shell.AbstractShell;
import org.ow2.petals.cli.shell.PetalsFileScriptShell;
import org.ow2.petals.cli.shell.extension.PetalsAdminShellExtension;

/**
 * Unit test of {@link Connect}
 *
 * @author Christophe DENEUX - Linagora
 *
 */
public class ConnectTest extends AbstractShellTest {

    private final static String ALIAS_IN_PREF_FILE = "test-alias";

    private final static String HOST_IN_PREF_FILE = "localhost";

    private final static String PORT_IN_PREF_FILE = "7700";

    @Rule
    final public TemporaryPreferencesFile tempPrefFile = new TemporaryPreferencesFile();

    /**
     * Test that the command usage is defined
     */
    @Test
    public void testUsage_0() {
        final Connect command = new Connect();
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage", usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage", usage.contains(command.getName()));
    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void unrecognizedOption() throws CommandException {
        final DummyShell dummyShell = new DummyShell();
        final Connect command = new Connect();
        command.setShell(dummyShell);
        try {
            command.execute(new String[] { "-z" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue("Error label is missing.", e.getMessage()
                    .startsWith(CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>the JMX port has an alphanumeric value</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void jmxPortAsAlphanumeric() throws CommandException {
        final DummyShell dummyShell = new DummyShell();
        final Connect command = new Connect();
        command.setShell(dummyShell);
        final String expectedInvalidValue = "alphanum-value";
        try {
            command.execute(new String[] { "-" + Connect.HOST_SHORT_OPTION, "localhost",
                    "-" + Connect.PORT_SHORT_OPTION, expectedInvalidValue, "-" + Connect.USER_SHORT_OPTION, "petals",
                    "-" + Connect.PWD_SHORT_OPTION, "petals" });
            fail("CommandInvalidArgumentException is not thrown");
        } catch (final CommandInvalidArgumentException e) {
            assertCommandInvalidArgumentException(e, Connect.PORT_SHORT_OPTION, expectedInvalidValue);
        }
    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias without username/password (only host/port are defined),</li>
     * <li>without yes flag,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>username and password are asked,</li>
     * <li>after to have entered username and password, the command is executed displaying the prompt</li>
     * </ul>
     * </p>
     */
    @Test
    public void testArgumentsError_01() throws Exception {

        this.tempPrefFile.addAlias(
                new DefaultAlias(ALIAS_IN_PREF_FILE).setHostName(HOST_IN_PREF_FILE).setPort(PORT_IN_PREF_FILE));

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            runCli("-C");

            // Wait that Petals CLI starts
            checkOut(containsString(Main.PROMPT_BASE + ">"));

            // Enter the command
            systemIn.writeLine("connect -a " + ALIAS_IN_PREF_FILE);

            // Wait the execution of the command
            checkOut(endsWith(Connect.USERNAME_QUESTION));

            // Enter username
            systemIn.writeLine("petals");

            // Wait the execution of the command
            checkOut(endsWith(Connect.PWD_QUESTION));

            // Enter password
            systemIn.writeLine("petals");

            // Wait the execution of the command
            checkOut(endsWith(Main.PROMPT_BASE + "@" + HOST_IN_PREF_FILE + ":" + PORT_IN_PREF_FILE + ">"));

            systemIn.writeLine("exit");

            waitThreadOrFail();

        } finally {
            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias without username/password (only host/port are defined),</li>
     * <li>with yes flag as command option,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>username and password are not asked,</li>
     * <li>an error occurs about missing username and password</li>
     * </ul>
     * </p>
     */
    @Test
    public void testArgumentsError_02() throws Exception {

        this.tempPrefFile.addAlias(
                new DefaultAlias(ALIAS_IN_PREF_FILE).setHostName(HOST_IN_PREF_FILE).setPort(PORT_IN_PREF_FILE));

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            runCli("-C");

            // Wait that Petals CLI starts
            checkOut(containsString(Main.PROMPT_BASE + ">"));

            // Enter the command
            systemIn.writeLine("connect -a " + ALIAS_IN_PREF_FILE + " -y");

            // Wait the execution of the command
            checkErrEndsWith(Constants.CommandMessages.MISSING_USERNAME_AND_PASSWORD);
            assertTrue("Petals CLI is connected", systemOut.getLog().contains(Main.PROMPT_BASE + ">"));

            systemIn.writeLine("exit");

            waitThreadOrFail();

            if (this.assertion != null) {
                throw this.assertion;
            }
        } finally {
            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias without username/password (only host/port are defined),</li>
     * <li>with yes flag on the command line,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>username and password are not asked,</li>
     * <li>an error occurs about missing username and password</li>
     * </ul>
     * </p>
     */
    @Test
    public void testArgumentsError_03() throws Exception {

        this.tempPrefFile.addAlias(
                new DefaultAlias(ALIAS_IN_PREF_FILE).setHostName(HOST_IN_PREF_FILE).setPort(PORT_IN_PREF_FILE));

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            runCli("-y", "-C");
            // Wait that Petals CLI starts
            systemIn.writeLine("connect -a " + ALIAS_IN_PREF_FILE);

            // Wait the execution of the command
            checkErrEndsWith(Constants.CommandMessages.MISSING_USERNAME_AND_PASSWORD);
            assertTrue("Petals CLI is connected", systemOut.getLog().contains(Main.PROMPT_BASE + ">"));

            systemIn.writeLine("exit");

            waitThreadOrFail();

            if (this.assertion != null) {
                throw this.assertion;
            }
        } finally {
            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias with a username and without password (host/port are also defined),</li>
     * <li>without yes flag,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>an error occurs about the missing password and it is not asked</li>
     * <li>the interactive shell ends,</li>
     * <li>the exit code is correctly set to 1.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testArgumentsError_07() throws Exception {

        this.tempPrefFile.addAlias(new DefaultAlias(ALIAS_IN_PREF_FILE).setHostName(HOST_IN_PREF_FILE)
                .setPort(PORT_IN_PREF_FILE).setJmxUser("petals"));

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            final Runnable petalsCli = new Runnable() {

                @Override
                public void run() {

                    try {
                        final int exitCode = new Main().run(new String[] { "-C" }, new PreferencesImpl(), null);

                        assertEquals("invalid exit code", 1, exitCode);
                    } catch (final AssertionError e) {
                        ConnectTest.this.assertion = e;
                    }

                }
            };

            final Thread petalsCliThread = new Thread(petalsCli, "petalsCliThread");
            petalsCliThread.start();

            // Wait that Petals CLI starts
            checkErr(containsString(ALIAS_IN_PREF_FILE));
            assertFalse("Prompt is displayed", systemOut.getLog().contains(Main.PROMPT_BASE + ">"));

            petalsCliThread.join();

            if (this.assertion != null) {
                throw this.assertion;
            }
        } finally {
            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias without a username and with a password (host/port are also defined),</li>
     * <li>without yes flag,</li>
     * <li>using an interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>an error occurs about the missing username and it is not asked</li>
     * <li>the interactive shell ends,</li>
     * <li>the exit code is correctly set to 1.</li>
     * </ul>
     * </p>
     */
    @Test
    public void testArgumentsError_08() throws Exception {

        this.tempPrefFile.addAlias(new DefaultAlias(ALIAS_IN_PREF_FILE).setHostName(HOST_IN_PREF_FILE)
                .setPort(PORT_IN_PREF_FILE).setJmxPwd("petals"));

        try {
            // Enable the hack for Windows?
            PetalsInteractiveCliUtils.hackForWindows(true);

            final Runnable petalsCli = new Runnable() {

                @Override
                public void run() {

                    try {
                        final int exitCode = new Main().run(new String[] { "-C" }, new PreferencesImpl(), null);

                        assertEquals("invalid exit code", 1, exitCode);
                    } catch (final AssertionError e) {
                        ConnectTest.this.assertion = e;
                    }

                }
            };

            final Thread petalsCliThread = new Thread(petalsCli, "petalsCliThread");
            petalsCliThread.start();

            // Wait that Petals CLI starts
            checkErr(containsString(ALIAS_IN_PREF_FILE));
            assertFalse("Prompt is displayed", systemOut.getLog().contains(Main.PROMPT_BASE + ">"));

            petalsCliThread.join();

            if (this.assertion != null) {
                throw this.assertion;
            }
        } finally {
            // Disable the hack for Windows
            PetalsInteractiveCliUtils.hackForWindows(false);
        }
    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias without username/password (only host/port are defined),</li>
     * <li>without yes flag,</li>
     * <li>using a non-interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about missing username and password.
     * </p>
     */
    @Test
    public void testArgumentsError_04() throws Exception {

        this.tempPrefFile.addAlias(
                new DefaultAlias(ALIAS_IN_PREF_FILE).setHostName(HOST_IN_PREF_FILE).setPort(PORT_IN_PREF_FILE));

        final Connect command = new Connect();

        // Create the Petals CLI script
        final File petalsCliScript = this.tempFolder.newFile("petals-cli.cmd");
        final OutputStream osScript = new FileOutputStream(petalsCliScript);
        final Writer pwScript = new OutputStreamWriter(osScript);
        try {
            pwScript.write(command.getName());
            pwScript.write(" -");
            pwScript.write(Connect.ALIAS_SHORT_OPTION);
            pwScript.write(ALIAS_IN_PREF_FILE);
            pwScript.write('\n');
        } finally {
            pwScript.close();
            osScript.close();
        }

        final Preferences pref = new PreferencesImpl();
        pref.intializePreferenceParameters();
        final PetalsFileScriptShell shell = new PetalsFileScriptShell(petalsCliScript.getAbsolutePath(), null, false,
                false, pref, null, null);
        shell.registersCommand(command);
        command.setShell(shell);

        shell.run();

        checkErr(containsString(Constants.CommandMessages.MISSING_USERNAME_AND_PASSWORD));

    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias without username/password (only host/port are defined),</li>
     * <li>with the yes flag set as command option,</li>
     * <li>using a non-interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about missing username and password.
     * </p>
     */
    @Test
    public void testArgumentsError_05() throws Exception {

        this.tempPrefFile.addAlias(
                new DefaultAlias(ALIAS_IN_PREF_FILE).setHostName(HOST_IN_PREF_FILE).setPort(PORT_IN_PREF_FILE));

        final Connect command = new Connect();

        // Create the Petals CLI script
        final File petalsCliScript = this.tempFolder.newFile("petals-cli.cmd");
        final OutputStream osScript = new FileOutputStream(petalsCliScript);
        final Writer pwScript = new OutputStreamWriter(osScript);
        try {
            pwScript.write(command.getName());
            pwScript.write(" -");
            pwScript.write(Connect.ALIAS_SHORT_OPTION);
            pwScript.write(ALIAS_IN_PREF_FILE);
            pwScript.write(" -");
            pwScript.write(Connect.YESFLAG_SHORT_OPTION);
            pwScript.write('\n');
        } finally {
            pwScript.close();
            osScript.close();
        }

        final Preferences pref = new PreferencesImpl();
        pref.intializePreferenceParameters();
        final PetalsFileScriptShell shell = new PetalsFileScriptShell(petalsCliScript.getAbsolutePath(), null, false,
                false, pref, null, null);
        shell.registersCommand(command);
        command.setShell(shell);

        shell.run();

        checkErr(containsString(Constants.CommandMessages.MISSING_USERNAME_AND_PASSWORD));

    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias without username/password (only host/port are defined),</li>
     * <li>with the yes flag set on the command line,</li>
     * <li>using a non-interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about missing username and password.
     * </p>
     */
    @Test
    public void testArgumentsError_06() throws Exception {

        this.tempPrefFile.addAlias(
                new DefaultAlias(ALIAS_IN_PREF_FILE).setHostName(HOST_IN_PREF_FILE).setPort(PORT_IN_PREF_FILE));

        final Connect command = new Connect();

        // Create the Petals CLI script
        final File petalsCliScript = this.tempFolder.newFile("petals-cli.cmd");
        final OutputStream osScript = new FileOutputStream(petalsCliScript);
        final Writer pwScript = new OutputStreamWriter(osScript);
        try {
            pwScript.write(command.getName());
            pwScript.write(" -");
            pwScript.write(Connect.ALIAS_SHORT_OPTION);
            pwScript.write(ALIAS_IN_PREF_FILE);
            pwScript.write('\n');
        } finally {
            pwScript.close();
            osScript.close();
        }

        final Preferences pref = new PreferencesImpl();
        pref.intializePreferenceParameters();
        final PetalsFileScriptShell shell = new PetalsFileScriptShell(petalsCliScript.getAbsolutePath(), null, false,
                true, pref, null, null);
        shell.registersCommand(command);
        command.setShell(shell);

        shell.run();

        checkErr(containsString(Constants.CommandMessages.MISSING_USERNAME_AND_PASSWORD));

    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias with username/password (host/port are also defined),</li>
     * <li>without yes flag,</li>
     * <li>using a non-interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs.
     * </p>
     */
    @Test
    public void testArguments_00() throws Exception {

        this.tempPrefFile.addAlias(new DefaultAlias(ALIAS_IN_PREF_FILE).setHostName(HOST_IN_PREF_FILE)
                .setPort(PORT_IN_PREF_FILE).setJmxUser("petals").setJmxPwd("petals"));

        final Connect command = new Connect();

        // Create the Petals CLI script
        final File petalsCliScript = this.tempFolder.newFile("petals-cli.cmd");
        final OutputStream osScript = new FileOutputStream(petalsCliScript);
        final Writer pwScript = new OutputStreamWriter(osScript);
        try {
            pwScript.write(command.getName());
            pwScript.write(" -");
            pwScript.write(Connect.ALIAS_SHORT_OPTION);
            pwScript.write(ALIAS_IN_PREF_FILE);
            pwScript.write('\n');
        } finally {
            pwScript.close();
            osScript.close();
        }

        final Preferences pref = new PreferencesImpl();
        pref.intializePreferenceParameters();
        final PetalsFileScriptShell shell = new PetalsFileScriptShell(petalsCliScript.getAbsolutePath(), null, false,
                false, pref, new ShellExtension[] { new PetalsAdminShellExtension() }, null);
        shell.registersCommand(command);
        command.setShell(shell);

        shell.run();

        assertTrue("An error message is printed", systemErr.getLog().isEmpty());
        assertEquals("Unexpected exit code", 0, shell.getExitStatus());

    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias with username/password (host/port are also defined),</li>
     * <li>with a yes flag set as command option,</li>
     * <li>using a non-interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs.
     * </p>
     */
    @Test
    public void testArguments_01() throws Exception {

        this.tempPrefFile.addAlias(new DefaultAlias(ALIAS_IN_PREF_FILE).setHostName(HOST_IN_PREF_FILE)
                .setPort(PORT_IN_PREF_FILE).setJmxUser("petals").setJmxPwd("petals"));

        final Connect command = new Connect();

        // Create the Petals CLI script
        final File petalsCliScript = this.tempFolder.newFile("petals-cli.cmd");
        final OutputStream osScript = new FileOutputStream(petalsCliScript);
        final Writer pwScript = new OutputStreamWriter(osScript);
        try {
            pwScript.write(command.getName());
            pwScript.write(" -");
            pwScript.write(Connect.ALIAS_SHORT_OPTION);
            pwScript.write(ALIAS_IN_PREF_FILE);
            pwScript.write(" -");
            pwScript.write(Connect.YESFLAG_SHORT_OPTION);
            pwScript.write('\n');
        } finally {
            pwScript.close();
            osScript.close();
        }

        final Preferences pref = new PreferencesImpl();
        pref.intializePreferenceParameters();
        final PetalsFileScriptShell shell = new PetalsFileScriptShell(petalsCliScript.getAbsolutePath(), null, false,
                false, pref, new ShellExtension[] { new PetalsAdminShellExtension() }, null);
        shell.registersCommand(command);
        command.setShell(shell);

        shell.run();

        assertTrue("An error message is printed", systemErr.getLog().isEmpty());
        assertEquals("Unexpected exit code", 0, shell.getExitStatus());

    }

    /**
     * Test of the command "connect" against argument management, where:
     * <ul>
     * <li>using an alias with username/password (host/port are also defined),</li>
     * <li>with the yes flag set on the command line,</li>
     * <li>using a non-interactive shell.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: no error occurs.
     * </p>
     */
    @Test
    public void testArguments_02() throws Exception {

        this.tempPrefFile.addAlias(new DefaultAlias(ALIAS_IN_PREF_FILE).setHostName(HOST_IN_PREF_FILE)
                .setPort(PORT_IN_PREF_FILE).setJmxUser("petals").setJmxPwd("petals"));

        final Connect command = new Connect();

        // Create the Petals CLI script
        final File petalsCliScript = this.tempFolder.newFile("petals-cli.cmd");
        final OutputStream osScript = new FileOutputStream(petalsCliScript);
        final Writer pwScript = new OutputStreamWriter(osScript);
        try {
            pwScript.write(command.getName());
            pwScript.write(" -");
            pwScript.write(Connect.ALIAS_SHORT_OPTION);
            pwScript.write(ALIAS_IN_PREF_FILE);
            pwScript.write('\n');
        } finally {
            pwScript.close();
            osScript.close();
        }

        final Preferences pref = new PreferencesImpl();
        pref.intializePreferenceParameters();
        final PetalsFileScriptShell shell = new PetalsFileScriptShell(petalsCliScript.getAbsolutePath(), null, false,
                true, pref, new ShellExtension[] { new PetalsAdminShellExtension() }, null);
        shell.registersCommand(command);
        command.setShell(shell);

        shell.run();

        assertTrue("An error message is printed", systemErr.getLog().isEmpty());
        assertEquals("Unexpected exit code", 0, shell.getExitStatus());

    }

    private static final String DEFAULT_HOST = "1.1.1.1";

    private static final int DEFAULT_PORT = 4875;

    private static final String DEFAULT_ALIAS = "default";

    private static final String ALIAS_1 = "alias-1";

    private static final String ALIAS_1_HOST = "localhost";

    private static final int ALIAS_1_PORT = 7845;

    private static final String ALIAS_3 = "alias-3";

    private static final String ALIAS_3_HOST = "2.2.2.2";

    private static final int ALIAS_3_PORT = 4123;

    private static final String ALIAS_3_USER = "testUser";

    private static final String ALIAS_3_PWD = "testPassword";

    private void createPreferenceFile()
            throws IOException, AliasPropertyAlreadySetException, DefaultAliasAlreadyExistsException {

        this.tempPrefFile
                .addAlias(
                        new DefaultAlias(DEFAULT_ALIAS).setHostName(DEFAULT_HOST).setPort(String.valueOf(DEFAULT_PORT)))
                .addAlias(new Alias(ALIAS_1).setHostName(ALIAS_1_HOST).setPort(String.valueOf(ALIAS_1_PORT)))
                .addAlias(new Alias(ALIAS_3).setHostName(ALIAS_3_HOST).setPort(String.valueOf(ALIAS_3_PORT))
                        .setJmxUser(ALIAS_3_USER).setJmxPwd(ALIAS_3_PWD));
    }

    @Test
    public final void testParseConnectionParametersWithNothingAndUseDefault() throws Exception {

        this.createPreferenceFile();

        // TODO this is very slow because the hostname is being resolved...
        final Connect command = new Connect();
        final Preferences pref = new PreferencesImpl();
        pref.intializePreferenceParameters();
        command.setShell(new AbstractShell(System.out, System.err, false, false, pref, null, "") {
            @Override
            public void run() {
                // NOP
            }
        });
        final CommandLineParser parser = new DefaultParser();
        final CommandLine cmd = parser.parse(command.getOptions(), new String[] { command.getName() });

        final ConnectionParameters connectionParameters = command.parseConnectionParameters(cmd, true);
        assertNotNull(connectionParameters);
        assertEquals(new ConnectionParameters(DEFAULT_HOST, DEFAULT_PORT), connectionParameters);
    }

    @Test
    public final void testParseConnectionParametersWithNothingNotUseDefault() throws Exception {

        this.createPreferenceFile();

        final Connect command = new Connect();
        command.setShell(new AbstractShell(System.out, System.err, false, false, new PreferencesImpl(), null, "") {
            @Override
            public void run() {
                // NOP
            }
        });
        final CommandLineParser parser = new DefaultParser();
        final CommandLine cmd = parser.parse(command.getOptions(), new String[] { command.getName() });

        final ConnectionParameters connectionParameters = command.parseConnectionParameters(cmd, false);
        assertNull(connectionParameters);
    }

    @Test
    public final void testParseConnectionParametersWithAlias() throws Exception {

        this.createPreferenceFile();

        // TODO this is very slow because the hostname is being resolved...
        final Connect command = new Connect();
        final Preferences pref = new PreferencesImpl();
        pref.intializePreferenceParameters();
        command.setShell(new AbstractShell(System.out, System.err, false, false, pref, null, "") {
            @Override
            public void run() {
                // NOP
            }
        });
        final CommandLineParser parser = new DefaultParser();
        final CommandLine cmd = parser.parse(command.getOptions(),
                new String[] { command.getName(), "-" + Connect.ALIAS_SHORT_OPTION, ALIAS_3 });

        final ConnectionParameters connectionParameters = command.parseConnectionParameters(cmd, true);
        assertNotNull(connectionParameters);
        assertEquals(
                new AuthenticatedConnectionParametersImpl(ALIAS_3_HOST, ALIAS_3_PORT, ALIAS_3_USER, ALIAS_3_PWD, null),
                connectionParameters);
    }

    @Test
    public final void testParseConnectionParametersWithParameters() throws Exception {

        this.createPreferenceFile();

        final String expectedHost = "localhost";
        final int expectedPort = 7700;
        final String expectedUser = "user";
        final String expectedPwd = "pwd";

        final Connect command = new Connect();
        command.setShell(new AbstractShell(System.out, System.err, false, false, new PreferencesImpl(), null, "") {
            @Override
            public void run() {
                // NOP
            }
        });
        final CommandLineParser parser = new DefaultParser();
        final CommandLine cmd = parser.parse(command.getOptions(),
                new String[] { command.getName(), "-" + Connect.HOST_SHORT_OPTION, expectedHost,
                        "-" + Connect.PORT_SHORT_OPTION, String.valueOf(expectedPort), "-" + Connect.USER_SHORT_OPTION,
                        expectedUser, "-" + Connect.PWD_SHORT_OPTION, expectedPwd });

        final ConnectionParameters connectionParameters = command.parseConnectionParameters(cmd, true);
        assertNotNull(connectionParameters);
        assertEquals(
                new AuthenticatedConnectionParametersImpl(expectedHost, expectedPort, expectedUser, expectedPwd, null),
                connectionParameters);
    }

    @Test(expected = CommandException.class)
    public final void testParseConnectionParametersWithIncompatibleArguments() throws Exception {

        this.createPreferenceFile();

        final String expectedHost = "localhost";
        final int expectedPort = 7700;
        final String expectedUser = "user";
        final String expectedPwd = "pwd";

        final Connect command = new Connect();
        command.setShell(new AbstractShell(System.out, System.err, false, false, new PreferencesImpl(), null, "") {
            @Override
            public void run() {
                // NOP
            }
        });
        final CommandLineParser parser = new DefaultParser();
        final CommandLine cmd = parser.parse(command.getOptions(),
                new String[] { command.getName(), "-" + Connect.HOST_SHORT_OPTION, expectedHost,
                        "-" + Connect.PORT_SHORT_OPTION, String.valueOf(expectedPort), "-" + Connect.USER_SHORT_OPTION,
                        expectedUser, "-" + Connect.PWD_SHORT_OPTION, expectedPwd, "-" + Connect.ALIAS_SHORT_OPTION,
                        ALIAS_1 });

        try {
            command.parseConnectionParameters(cmd, true);
        } catch (final CommandException ce) {
            assertEquals(CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS,
                    ce.getMessage());
            throw ce;
        }
    }

    @Test(expected = CommandException.class)
    public final void testParseConnectionParametersWithBadNumberOfArguments() throws Exception {

        this.createPreferenceFile();

        final String expectedHost = "localhost";
        final int expectedPort = 7700;

        final Connect command = new Connect();
        command.setShell(new AbstractShell(System.out, System.err, false, false, new PreferencesImpl(), null, "") {
            @Override
            public void run() {
                // NOP
            }
        });
        final CommandLineParser parser = new DefaultParser();
        final CommandLine cmd = parser.parse(command.getOptions(),
                new String[] { command.getName(), "-" + Connect.HOST_SHORT_OPTION, expectedHost,
                        "-" + Connect.PORT_SHORT_OPTION, String.valueOf(expectedPort) });

        try {
            command.parseConnectionParameters(cmd, true);
        } catch (final CommandException ce) {
            assertEquals(CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS,
                    ce.getMessage());
            throw ce;
        }
    }
}
