/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.cli.Constants.CommonOption.URL_LONG_OPTION;
import static org.ow2.petals.cli.base.junit.Assert.assertCommandMissingOptionsException;
import static org.ow2.petals.cli.shell.command.Deploy.BULK_SHORT_OPTION;
import static org.ow2.petals.cli.shell.command.Deploy.SKIP_START_LONG_OPTION;

import java.io.File;
import java.net.URL;

import org.junit.Test;
import org.ow2.petals.admin.api.artifact.Artifact;
import org.ow2.petals.admin.api.artifact.ArtifactState;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandMissingOptionsException;
import org.ow2.petals.cli.api.command.exception.CommandTooManyArgumentsException;
import org.ow2.petals.cli.api.command.exception.CommandUnparsableArgumentException;
import org.ow2.petals.cli.api.command.exception.ConnectionRequiredException;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;

/**
 * 
 * @author "Mathieu CARROLLE - EBM WebSourcing"
 * 
 */
public class DeployTest extends AbstractArtifactCommandTest {

    /**
     * Test that the command usage is defined
     */
    @Test
    public void testUsage_0() {
        final Deploy command = new Deploy();
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage", usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage", usage.contains(command.getName()));
    }

    /**
     * Test of the command "deploy" against argument management, where:
     * <ul>
     * <li>no option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     * 
     * @see <a href="http://jira.petalslink.com/browse/PETALSESBCLI-45">PETALSESBCLI-45</a>
     */
    @Test
    public void testArgumentsError_0() throws Exception {

        final Deploy command = new Deploy();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] {});
            fail("CommandMissingOptionsException is not thrown");
        } catch (final CommandMissingOptionsException e) {
            assertCommandMissingOptionsException(e, Deploy.BULK_SHORT_OPTION,
                    org.ow2.petals.cli.Constants.CommonOption.URL_SHORT_OPTION);
        }
    }

    /**
     * Test of the command "deploy" against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_1() throws Exception {

        final Deploy command = new Deploy();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-z" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue("Error label is missing.", e.getMessage()
                    .startsWith(CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    @Test()
    public void testDeployOneArtifactFromFile() throws Exception {

        Deploy command = new Deploy();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "--" + URL_LONG_OPTION, new File(COMPONENT_URL.toURI()).getAbsolutePath() });
        assertEquals(1, getArtifactAdmin().listArtifacts().size());

    }

    @Test()
    public void testDeployOneArtifactFromFileWithTilde() throws Exception {
        Deploy command = new Deploy();
        command.setShell(this.dummyShellWrapper.getShell());
        final String fileWithTilde = new File(COMPONENT_URL.toURI()).getAbsolutePath()
                .replaceFirst(System.getProperty("user.home"), "~");
        command.execute(new String[] { "--" + URL_LONG_OPTION, fileWithTilde });
        assertEquals(1, getArtifactAdmin().listArtifacts().size());
    }

    @Test()
    public void testDeployOneArtifactFromFileURL() throws Exception {

        Deploy command = new Deploy();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "--" + URL_LONG_OPTION, COMPONENT_URL.toString() });
        assertEquals(1, getArtifactAdmin().listArtifacts().size());
    }

    @Test()
    public void testDeployOneArtifactFromMavenURL() throws Exception {
        Deploy command = new Deploy();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "--" + URL_LONG_OPTION, "mvn:org.ow2.petals/petals-bc-soap/4.4.2/zip" });
        assertEquals(1, getArtifactAdmin().listArtifacts().size());
    }

    @Test()
    public void testDeployOneArtifactFromURLWithoutStart() throws Exception {
        Deploy command = new Deploy();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(
                new String[] { "--" + URL_LONG_OPTION, COMPONENT_URL.toString(), "--" + SKIP_START_LONG_OPTION });
        assertEquals(1, getArtifactAdmin().listArtifacts().size());
        final Artifact artifact = getArtifactAdmin().listArtifacts().get(0);
        assertTrue(artifact instanceof ArtifactState);
        assertTrue(((ArtifactState) artifact).getState() != ArtifactState.State.STARTED);
    }

    @Test()
    public void testDeployOneArtifactFromURLWithInlinedPropertiesWithoutStart() throws Exception {
        Deploy command = new Deploy();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "--" + URL_LONG_OPTION, COMPONENT_URL.toString(), "--" + SKIP_START_LONG_OPTION,
                "-D", "key1=value1,key2=value2" });
        assertEquals(1, getArtifactAdmin().listArtifacts().size());
        final Artifact artifact = getArtifactAdmin().listArtifacts().get(0);
        assertTrue(artifact instanceof ArtifactState);
        assertTrue(((ArtifactState) artifact).getState() != ArtifactState.State.STARTED);
    }

    @Test
    public void testErrorWithMoreThanOneURL() throws Exception {

        Deploy command = new Deploy();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "--" + URL_LONG_OPTION, COMPONENT_URL.toString(), SA_URL.toString() });

            fail("Exception 'CommandTooManyArgumentsException' is not thrown");
        } catch (final CommandTooManyArgumentsException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandTooManyArgumentsException.TOO_MANY_ARGUMENTS));
        }
    }

    @Test()
    public void testDeployOneArtifactFromURLWithPropertiesFile() throws Exception {
        final URL propertiesFile = Thread.currentThread().getContextClassLoader().getResource("test.properties");

        Deploy command = new Deploy();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(
                new String[] { "--" + URL_LONG_OPTION, COMPONENT_URL.toString(), "-f", propertiesFile.toString() });
        assertEquals(1, getArtifactAdmin().listArtifacts().size());
    }

    @Test()
    public void testDeployOneArtifactFromURLWithInlinedProperties() throws Exception {

        Deploy command = new Deploy();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(
                new String[] { "--" + URL_LONG_OPTION, COMPONENT_URL.toString(), "-D", "key1=value1,key2=value2" });
        assertEquals(1, getArtifactAdmin().listArtifacts().size());
    }

    /**
     * Test of the command "deploy" against argument management, where:
     * <ul>
     * <li>the url of the artifact (component) to deploy is correctly provided,</li>
     * <li>configuration properties option is set without value.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number
     * </p>
     */
    @Test
    public void testArgumentsError_2() throws Exception {

        final Deploy command = new Deploy();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "--" + URL_LONG_OPTION, COMPONENT_URL.toString(),
                    "-" + Deploy.INLINED_PROPERTIES_SHORT_OPTION });
            fail("CommandMissingArgumentException is not thrown");
        } catch (final CommandMissingArgumentException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandMissingArgumentException.MISSING_ARGUMENT));
            assertTrue("The option '-" + Deploy.INLINED_PROPERTIES_SHORT_OPTION + "' has a missing argument.",
                    Deploy.INLINED_PROPERTIES_SHORT_OPTION.equals(e.getOption().getOpt()));
            assertTrue("Missing argument is missing in error message.",
                    e.getMessage().endsWith(Deploy.INLINED_PROPERTIES_SHORT_OPTION));
        }
    }

    /**
     * Test of the command "deploy" against argument management, where:
     * <ul>
     * <li>the url of the artifact (component) to deploy is correctly provided,</li>
     * <li>configuration properties option is set with value containing only a property name, no property value.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number
     * </p>
     */
    @Test
    public void testArgumentsError_3() throws Exception {
        final String unparsableProperty = "key2;value2";
        final Deploy command = new Deploy();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "--" + URL_LONG_OPTION, COMPONENT_URL.toString(),
                    "-" + Deploy.INLINED_PROPERTIES_SHORT_OPTION, "key1=value1," + unparsableProperty });
            fail("CommandUnparsableArgumentException is not thrown");
        } catch (final CommandUnparsableArgumentException e) {
            assertTrue("The option is missing.", e.getMessage().contains("-" + Deploy.INLINED_PROPERTIES_SHORT_OPTION));
            assertTrue("The unparsable value is missing.", e.getMessage().contains(unparsableProperty));
        }
    }

    @Test()
    public void testBulkDeployFromFile() throws Exception {

        final Deploy command = new Deploy();
        command.setShell(this.dummyShellWrapper.getShell());
        final File file = new File(COMPONENT_URL.toURI()).getParentFile();
        assertTrue(file.isDirectory());
        command.execute(new String[] { "-" + BULK_SHORT_OPTION, file.getPath() });
        assertEquals(22, getArtifactAdmin().listArtifacts().size());
    }

    @Test()
    public void testBulkDeployFromFileWithTilde() throws Exception {
        final Deploy command = new Deploy();
        command.setShell(this.dummyShellWrapper.getShell());
        final File file = new File(COMPONENT_URL.toURI()).getParentFile();
        assertTrue(file.isDirectory());
        final String fileWithTilde = file.getPath().replaceFirst(System.getProperty("user.home"), "~");
        command.execute(new String[] { "-" + BULK_SHORT_OPTION, fileWithTilde });
        assertEquals(22, getArtifactAdmin().listArtifacts().size());
    }

    @Test()
    public void testBulkDeployFromURL() throws Exception {
        final Deploy command = new Deploy();
        command.setShell(this.dummyShellWrapper.getShell());
        final File file = new File(COMPONENT_URL.toURI()).getParentFile();
        assertTrue(file.isDirectory());
        command.execute(new String[] { "-" + BULK_SHORT_OPTION, file.toURI().toURL().toString() });
        assertEquals(22, getArtifactAdmin().listArtifacts().size());
    }

    @Test(expected = CommandException.class)
    public void testBulkDeployWithNonFileTypeURL() throws Exception {

        Deploy command = new Deploy();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + BULK_SHORT_OPTION, "http://myfile.zip" });
        } catch (CommandException e) {
            assertEquals("Directory deployment is only allowed for file protocol", e.getMessage());
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "deploy" when no connection is established to run the command.
     * </p>
     * <p>
     * Expected results: an error occurs about the connection not established.
     * </p>
     */
    @Test(expected = ConnectionRequiredException.class)
    public void executeNotConnected() throws CommandException {

        final StringStreamShell shell = new StringStreamShell();
        final Command command = new Deploy();
        command.setShell(shell);

        command.execute(new String[] {});
    }

}
