/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import java.util.Set;
import java.util.TreeSet;

import org.junit.Test;
import org.ow2.petals.admin.api.artifact.Artifact;
import org.ow2.petals.admin.api.artifact.ArtifactState;
import org.ow2.petals.admin.api.artifact.ArtifactUtils;
import org.ow2.petals.admin.api.artifact.ServiceAssembly;
import org.ow2.petals.admin.api.artifact.ServiceUnit;
import org.ow2.petals.admin.api.artifact.SharedLibrary;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.ConnectionRequiredException;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;

/**
 * 
 * @author "Mathieu CARROLLE - EBM WebSourcing"
 * 
 */
public class ListArtifactsTest extends AbstractArtifactCommandTest {

    /**
     * Test that the command usage is defined
     */
    @Test
    public void testUsage_0() {
        final ListArtifacts command = new ListArtifacts();
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage", usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage", usage.contains(command.getName()));
    }

    /**
     * <p>
     * Test of the command "list" to list deployed artifact against no deployed artifact.
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs</li>
     * <li>the output is empty</li>
     * </ul>
     * </p>
     */
    @Test
    public void testExecuteWithoutArtifactsInRegistry() throws Exception {

        final ListArtifacts command = new ListArtifacts();
        command.setShell(this.dummyShellWrapper.getShell());

        command.execute(new String[] {});
        assertEquals("", this.dummyShellWrapper.getOut().toString());
    }

    @Test
    public void testExecuteWithTwoArtifactsInRegistry() throws Exception {
        this.deployStartArtifact(this.slURL, COMPONENT_URL, SA_URL);

        final Artifact slArtifact = ArtifactUtils.createArtifact(this.slURL);
        final String componentName = ArtifactUtils.createArtifact(COMPONENT_URL).getName();
        final ServiceAssembly sa = (ServiceAssembly) ArtifactUtils.createArtifact(SA_URL);
        final String saName = sa.getName();

        final ListArtifacts command = new ListArtifacts();
        command.setShell(this.dummyShellWrapper.getShell());

        command.execute(new String[] {});
        final StringBuilder expected = new StringBuilder(slArtifact.getName() + "(" + slArtifact.getVersion() + ") SL\n"
                + componentName + " " + ArtifactState.State.STARTED.toString() + " BC\n" + saName + " "
                + ArtifactState.State.STARTED.toString() + " SA\n");

        final Set<ServiceUnit> serviceUnits = new TreeSet<>(command.artifactNameComparator);
        serviceUnits.addAll(sa.getServiceUnits());
        for (final ServiceUnit su : serviceUnits) {
            expected.append(su.getName() + " SU\n");
        }
        assertEquals(expected.toString(), this.dummyShellWrapper.getOut().toString());
    }

    /**
     * <p>
     * Test of the command "list" to list deployed artifact against bad number arguments, where the artifact regexp is
     * provided without the option: <code>list .*(EIP|eip|soap).*</code>.
     * </p>
     * <p>
     * Expected results: an error occurs about the bad argument number.
     * </p>
     */
    @Test
    public void testListArtifactCmdLineError_0() throws Exception {

        final ListArtifacts command = new ListArtifacts();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { ".*(EIP|eip|soap).*" });
            fail("CommandBadArgumentNumberException is not thrown.");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue("Error label is missing.", e.getMessage()
                    .startsWith(CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }

    }

    /**
     * <p>
     * Test of the command "list" to list deployed artifact against bad number arguments, where the artifact type is
     * provided without the option: <code>list SU</code>.
     * </p>
     * <p>
     * Expected results: an error occurs about the bad argument number.
     * </p>
     */
    @Test
    public void testListArtifactCmdLineError_1() throws Exception {

        final ListArtifacts command = new ListArtifacts();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "SU" });
            fail("CommandBadArgumentNumberException is not thrown.");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue("Error label is missing.", e.getMessage()
                    .startsWith(CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }

    }

    /**
     * <p>
     * Test of the command "list" to list deployed artifact against bad number arguments, where the artifact pattern is
     * correctly provided and the artifact type is provided without the option:
     * <code>list -p .*(EIP|eip|soap).* SU</code>.
     * </p>
     * <p>
     * Expected results: an error occurs about the bad argument number.
     * </p>
     */
    @Test
    public void testListArtifactCmdLineError_2() throws Exception {

        final ListArtifacts command = new ListArtifacts();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(
                    new String[] { "-" + ListArtifacts.REGEX_IDENTIFIER_SHORT_OPTION, ".*(EIP|eip|soap).*", "SU" });
            fail("CommandBadArgumentNumberException is not thrown.");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue("Error label is missing.", e.getMessage()
                    .startsWith(CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }

    }

    /**
     * <p>
     * Test of the command "list" to list deployed artifact against bad number arguments, where the artifact type is
     * correctly provided and the artifact pattern is provided without the option:
     * <code>list -t SU .*(EIP|eip|soap).*</code>.
     * </p>
     * <p>
     * Expected results: an error occurs about the bad argument number.
     * </p>
     */
    @Test
    public void testListArtifactCmdLineError_3() throws Exception {

        final ListArtifacts command = new ListArtifacts();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + ListArtifacts.TYPE_SHORT_OPTION, "SU", ".*(EIP|eip|soap).*" });
            fail("CommandBadArgumentNumberException is not thrown.");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue("Error label is missing.", e.getMessage()
                    .startsWith(CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }

    }

    /**
     * <p>
     * Test of the command "list" to list deployed artifact against bad number arguments, where the artifact pattern is
     * provided without the option and the artifact type is correctly provided:
     * <code>list .*(EIP|eip|soap).* -t SU </code>.
     * </p>
     * <p>
     * Expected results: an error occurs about the bad argument number.
     * </p>
     */
    @Test
    public void testListArtifactCmdLineError_4() throws Exception {

        final ListArtifacts command = new ListArtifacts();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { ".*(EIP|eip|soap).*", "-" + ListArtifacts.TYPE_SHORT_OPTION, "SU" });
            fail("CommandBadArgumentNumberException is not thrown.");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue("Error label is missing.", e.getMessage()
                    .startsWith(CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }

    }

    /**
     * Test of the command "list" against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testListArtifactCmdLineError_5() throws Exception {

        final ListArtifacts command = new ListArtifacts();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-z" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue("Error label is missing.", e.getMessage()
                    .startsWith(CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test of the command "list" to list deployed artifact, where:
     * <ul>
     * <li>the artifact pattern is provided,</li>
     * <li>several JBI artifacts are deployed (1 SL, 1 BC matching the regexp, 1 SA including 8 SUs with 4 matching the
     * regex.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error is throw,</li>
     * <li>the 5 artifacts matching the regexp are listed,</li>
     * <li>the JBI component is displayed first, next the 4 artifacts are displayed in the alphabetical order</li>
     * </ul>
     * </p>
     */
    @Test
    public void testListArtifactWithRegexFilter_0() throws Exception {
        this.deployStartArtifact(slURL, COMPONENT_URL, SA_URL);

        String componentName = ArtifactUtils.createArtifact(COMPONENT_URL).getName();
        ServiceAssembly sa = (ServiceAssembly) ArtifactUtils.createArtifact(SA_URL);

        final ListArtifacts command = new ListArtifacts();
        command.setShell(this.dummyShellWrapper.getShell());

        command.execute(new String[] { "-" + ListArtifacts.REGEX_IDENTIFIER_SHORT_OPTION, ".*(EIP|eip|soap).*" });

        final String output = this.dummyShellWrapper.getOut().toString();
        final int idxComponent = output.indexOf(componentName);
        assertTrue("Component not found in the list", idxComponent > -1);

        final Set<ServiceUnit> serviceUnits = new TreeSet<>(command.artifactNameComparator);
        serviceUnits.addAll(sa.getServiceUnits());
        final int[] idxSu = new int[serviceUnits.size()];
        int i = 0;
        for (final ServiceUnit serviceUnit : serviceUnits) {
            if (serviceUnit.getName().contains("-EIP-")) {
                idxSu[i++] = output.indexOf(serviceUnit.getName());
                if (i == 1) {
                    assertTrue("SU is not at the right place", idxSu[0] > idxComponent);
                } else {
                    assertTrue("SU is not at the right place", idxSu[i - 1] > idxSu[i - 2]);
                }
            }
        }
        assertEquals("SU list has not the right size", 4, i);
    }

    @Test
    public void testListArtifactWithSAFilter() throws Exception {
        this.deployStartArtifact(slURL, COMPONENT_URL, SA_URL);

        ServiceAssembly sa = (ServiceAssembly) ArtifactUtils.createArtifact(SA_URL);

        final ListArtifacts command = new ListArtifacts();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "-" + ListArtifacts.TYPE_SHORT_OPTION, "SA" });

        assertEquals(sa.getName() + " " + ArtifactState.State.STARTED.toString() + " SA\n",
                this.dummyShellWrapper.getOut().toString());
    }

    @Test
    public void testListArtifactWithSUFilter() throws Exception {
        this.deployStartArtifact(slURL, COMPONENT_URL, SA_URL);

        ServiceAssembly sa = (ServiceAssembly) ArtifactUtils.createArtifact(SA_URL);

        final ListArtifacts command = new ListArtifacts();
        command.setShell(this.dummyShellWrapper.getShell());

        command.execute(new String[] { "-" + ListArtifacts.TYPE_SHORT_OPTION, "SU" });
        final StringBuilder expected = new StringBuilder();

        final Set<ServiceUnit> serviceUnits = new TreeSet<>(command.artifactNameComparator);
        serviceUnits.addAll(sa.getServiceUnits());
        for (final ServiceUnit su : serviceUnits) {
            expected.append(su.getName() + " SU\n");
        }
        assertEquals(expected.toString(), this.dummyShellWrapper.getOut().toString());
    }

    @Test
    public void testListArtifactWithBCFilter() throws Exception {
        this.deployStartArtifact(slURL, COMPONENT_URL, SA_URL);

        String componentName = ArtifactUtils.createArtifact(COMPONENT_URL).getName();
        final ListArtifacts command = new ListArtifacts();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "-" + ListArtifacts.TYPE_SHORT_OPTION, "BC" });

        assertEquals(componentName + " " + ArtifactState.State.STARTED.toString() + " BC\n",
                this.dummyShellWrapper.getOut().toString());
    }

    @Test
    public void testListArtifactWithSEFilter() throws Exception {
        this.deployStartArtifact(slURL, COMPONENT_URL, SA_URL);

        final ListArtifacts command = new ListArtifacts();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "-" + ListArtifacts.TYPE_SHORT_OPTION, "SE" });

        assertEquals("", this.dummyShellWrapper.getOut().toString());
    }

    @Test
    public void testListArtifactWithSLFilter() throws Exception {
        this.deployStartArtifact(slURL, COMPONENT_URL, SA_URL);

        final Artifact artifact = ArtifactUtils.createArtifact(this.slURL);
        final ListArtifacts command = new ListArtifacts();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "-" + ListArtifacts.TYPE_SHORT_OPTION, SharedLibrary.TYPE });

        assertEquals(artifact.getName() + "(" + artifact.getVersion() + ") SL\n",
                this.dummyShellWrapper.getOut().toString());
    }

    @Test
    public void testListArtifactWithSLVersionFilter() throws Exception {
        this.deployStartArtifact(slURL, sl2URL, COMPONENT_URL);

        final Artifact compArtifact = ArtifactUtils.createArtifact(COMPONENT_URL);
        final Artifact slArtifact = ArtifactUtils.createArtifact(this.slURL);
        ArtifactUtils.createArtifact(this.sl2URL);
        final ListArtifacts command = new ListArtifacts();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "-" + ListArtifacts.SL_VERSION_REGEX_SHORT_OPTION, "5\\..*" });

        assertEquals(slArtifact.getName() + "(" + slArtifact.getVersion() + ") SL\n" + compArtifact.getName() + " "
                + ArtifactState.State.STARTED.toString() + " BC\n", this.dummyShellWrapper.getOut().toString());
    }

    @Test
    public void testListArtifactWithTwoVersionsForSL() throws Exception {
        this.deployStartArtifact(slURL, sl2URL, COMPONENT_URL);

        final Artifact compArtifact = ArtifactUtils.createArtifact(COMPONENT_URL);
        final Artifact slArtifact = ArtifactUtils.createArtifact(this.slURL);
        final Artifact sl2Artifact = ArtifactUtils.createArtifact(this.sl2URL);
        final ListArtifacts command = new ListArtifacts();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] {});

        assertEquals(sl2Artifact.getName() + "(" + sl2Artifact.getVersion() + ") SL\n" + slArtifact.getName() + "("
                + slArtifact.getVersion() + ") SL\n" + compArtifact.getName() + " "
                + ArtifactState.State.STARTED.toString() + " BC\n", this.dummyShellWrapper.getOut().toString());
    }

    @Test
    public void testListArtifactWithSLAndVersionFilter() throws Exception {
        this.deployStartArtifact(slURL, sl2URL, COMPONENT_URL, SA_URL);

        final Artifact artifact = ArtifactUtils.createArtifact(this.slURL);
        final ListArtifacts command = new ListArtifacts();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "-" + ListArtifacts.TYPE_SHORT_OPTION, SharedLibrary.TYPE,
                "-" + ListArtifacts.SL_VERSION_REGEX_SHORT_OPTION, "5\\..*" });

        assertEquals(artifact.getName() + "(" + artifact.getVersion() + ") SL\n",
                this.dummyShellWrapper.getOut().toString());
    }

    @Test
    public void testListArtifactWithRegexAndArtifactTypeFilter() throws Exception {
        this.deployStartArtifact(slURL, COMPONENT_URL, SA_URL);

        final ServiceAssembly sa = (ServiceAssembly) ArtifactUtils.createArtifact(SA_URL);

        final ListArtifacts command = new ListArtifacts();
        command.setShell(this.dummyShellWrapper.getShell());

        command.execute(new String[] { "-" + ListArtifacts.REGEX_IDENTIFIER_SHORT_OPTION, ".*EIP.*",
                "-" + ListArtifacts.TYPE_SHORT_OPTION, "SU" });
        final StringBuilder expected = new StringBuilder();

        final Set<ServiceUnit> serviceUnits = new TreeSet<>(command.artifactNameComparator);
        serviceUnits.addAll(sa.getServiceUnits());
        for (final ServiceUnit su : serviceUnits) {
            if (su.getName().contains("EIP")) {
                expected.append(su.getName() + " SU\n");
            }
        }
        assertEquals(expected.toString(), this.dummyShellWrapper.getOut().toString());
    }

    /**
     * <p>
     * Test of the command "list" when no connection is established to run the command.
     * </p>
     * <p>
     * Expected results: an error occurs about the connection not established.
     * </p>
     */
    @Test(expected = ConnectionRequiredException.class)
    public void executeNotConnected() throws Exception {

        final StringStreamShell shell = new StringStreamShell();
        final Command command = new ListArtifacts();
        command.setShell(shell);

        command.execute(new String[] {});
    }
}
