/**
 * Copyright (c) 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;

import org.junit.Test;
import org.ow2.petals.admin.api.exception.EndpointDirectoryRegexpPatternException;
import org.ow2.petals.admin.junit.EndpointDirectoryViewMock;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.ConnectionRequiredException;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;

/**
 * Unit test of {@link ListEndpoints}
 * 
 * @author Christophe DENEUX - Linagora
 * 
 */
public class ListEndpointsTest extends AbstractConnectedShellTest {

    private final static String INVALID_REGEXP = "{http://petals.ow2.org/}FaultIntegrationItf";

    /**
     * Test that the command usage is defined
     */
    @Test
    public void testUsage_0() throws Exception {
        final Command command = new ListEndpoints();
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage", usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage", usage.contains(command.getName()));
    }

    /**
     * Test of the command "endpoint-list" against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_0() throws Exception {

        final Command command = new ListEndpoints();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-z" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    /**
     * Test of the command "endpoint-list" against argument management, where:
     * <ul>
     * <li>the regexp used to filter interfaces contains an error</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the regexp error.
     * </p>
     */
    @Test
    public void testArgumentsError_1() throws Exception {
        // TODO: prefer to deploy a SA to fill the endpoint directory instead of using
        // data defined in test classes petals-admin-api.

        final Command command = new ListEndpoints();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-i", INVALID_REGEXP });
            fail("EndpointDirectoryRegexpPatternException is not thrown through CommandException.");
        } catch (final CommandException e) {
            assertNotNull("CommandException cause is missing.", e.getCause());
            assertTrue("CommandException cause is not EndpointDirectoryRegexpPatternException",
                    e.getCause() instanceof EndpointDirectoryRegexpPatternException);
            assertTrue("Invalid regexp is not displayed", e.getMessage().contains(INVALID_REGEXP));
        }
    }

    /**
     * Test of the command "endpoint-list" against argument management, where:
     * <ul>
     * <li>the regexp used to filter services contains an error</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the regexp error.
     * </p>
     */
    @Test
    public void testArgumentsError_2() throws Exception {
        // TODO: prefer to deploy a SA to fill the endpoint directory instead of using
        // data defined in test classes petals-admin-api.

        final Command command = new ListEndpoints();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-s", INVALID_REGEXP });
            fail("EndpointDirectoryRegexpPatternException is not thrown through CommandException.");
        } catch (final CommandException e) {
            assertNotNull("CommandException cause is missing.", e.getCause());
            assertTrue("CommandException cause is not EndpointDirectoryRegexpPatternException",
                    e.getCause() instanceof EndpointDirectoryRegexpPatternException);
            assertTrue("Invalid regexp is not displayed", e.getMessage().contains(INVALID_REGEXP));
        }
    }

    /**
     * Test of the command "endpoint-list" against argument management, where:
     * <ul>
     * <li>the regexp used to filter endpoints contains an error</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about the regexp error.
     * </p>
     */
    @Test
    public void testArgumentsError_3() throws Exception {
        // TODO: prefer to deploy a SA to fill the endpoint directory instead of using
        // data defined in test classes petals-admin-api.

        final Command command = new ListEndpoints();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-e", INVALID_REGEXP });
            fail("EndpointDirectoryRegexpPatternException is not thrown through CommandException.");
        } catch (final CommandException e) {
            assertNotNull("CommandException cause is missing.", e.getCause());
            assertTrue("CommandException cause is not EndpointDirectoryRegexpPatternException",
                    e.getCause() instanceof EndpointDirectoryRegexpPatternException);
            assertTrue("Invalid regexp is not displayed", e.getMessage().contains(INVALID_REGEXP));
        }
    }

    /**
     * <p>
     * Check the command 'endpoint-list' against the output filtering where:
     * <ul>
     * <li>no output filtering</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs,</li>
     * <li>endpoints are displayed</li>
     * <li>the header of the section 'Endpoints' is displayed</li>
     * <li>services are displayed</li>
     * <li>the header of the section 'Services' is displayed</li>
     * <li>interfaces are displayed</li>
     * <li>the header of the section 'Interfaces' is displayed</li>
     * </ul>
     * </p>
     */
    @Test
    public void testNoOutputFiltering() throws Exception {
        // TODO: prefer to deploy a SA to fill the endpoint directory instead of using
        // data defined in test classes petals-admin-api.

        final Command command = new ListEndpoints();
        command.setShell(this.dummyShellWrapper.getShell());

        command.execute(new String[] {});

        final String output = this.dummyShellWrapper.getOut().toString();
        assertNotNull("No output", output);
        assertTrue("Empty output", !output.isEmpty());
        assertTrue("Header of section 'Endpoints' is not displayed", output.contains("Endpoints:"));
        assertTrue("Endpoint missing", output.contains(EndpointDirectoryViewMock.EP1_NAME));
        assertTrue("Header of section 'Services' is not displayed", output.contains("Services:"));
        assertTrue("Service missing", output.contains(EndpointDirectoryViewMock.EP1_SVC1_NAME));
        assertTrue("Header of section 'Interface' is not displayed", output.contains("Interfaces:"));
        assertTrue("Interface missing", output.contains(EndpointDirectoryViewMock.EP1_ITF1_NAME));
    }

    /**
     * <p>
     * Check the command 'endpoint-list' against the output filtering where:
     * <ul>
     * <li>only endpoints are asked to be displayed</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs,</li>
     * <li>only endpoints are displayed</li>
     * <li>the header of the section 'Endpoints' is not displayed</li>
     * </ul>
     * </p>
     */
    @Test
    public void testOutputFilteringLimitedToEndpoints() throws Exception {
        // TODO: prefer to deploy a SA to fill the endpoint directory instead of using
        // data defined in test classes petals-admin-api.

        final Command command = new ListEndpoints();
        command.setShell(this.dummyShellWrapper.getShell());

        command.execute(new String[] { "--" + ListEndpoints.OUTPUT_ENDPOINTS_LONG_OPTION });

        final String output = this.dummyShellWrapper.getOut().toString();
        assertNotNull("No output", output);
        assertTrue("Empty output", !output.isEmpty());
        assertTrue("Header of section 'Endpoints' displayed", !output.contains("Endpoints:"));
        assertTrue("Enpoint missing", output.startsWith(EndpointDirectoryViewMock.EP1_NAME));
        assertTrue("Output contains a service", !output.contains(EndpointDirectoryViewMock.EP1_SVC1_NAME));
        assertTrue("Output contains an interface", !output.contains(EndpointDirectoryViewMock.EP1_ITF1_NAME));
    }

    /**
     * <p>
     * Check the command 'endpoint-list' against the output filtering where:
     * <ul>
     * <li>only services are asked to be displayed</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs,</li>
     * <li>only services are displayed</li>
     * <li>the header of the section 'Services' is not displayed</li>
     * </ul>
     * </p>
     */
    @Test
    public void testOutputFilteringLimitedToServices() throws Exception {
        // TODO: prefer to deploy a SA to fill the endpoint directory instead of using
        // data defined in test classes petals-admin-api.

        final Command command = new ListEndpoints();
        command.setShell(this.dummyShellWrapper.getShell());

        command.execute(new String[] { "--" + ListEndpoints.OUTPUT_SERVICES_LONG_OPTION });

        final String output = this.dummyShellWrapper.getOut().toString();
        assertNotNull("No output", output);
        assertTrue("Empty output", !output.isEmpty());
        assertTrue("Header of section 'Services' displayed", !output.contains("Services:"));
        assertTrue("Service missing", output.startsWith(EndpointDirectoryViewMock.EP1_SVC1_NAME));
        assertTrue("Endpoint missing as sub element of service: " + output,
                output.endsWith(EndpointDirectoryViewMock.EP1_NAME + "\n"));
        assertTrue("Output contains an interface", !output.contains(EndpointDirectoryViewMock.EP1_ITF1_NAME));
    }

    /**
     * <p>
     * Check the command 'endpoint-list' against the output filtering where:
     * <ul>
     * <li>only interfaces are asked to be displayed</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs,</li>
     * <li>only interfaces are displayed</li>
     * <li>the header of the section 'Interfaces' is not displayed</li>
     * </ul>
     * </p>
     */
    @Test
    public void testOutputFilteringLimitedToInterfaces() throws Exception {
        // TODO: prefer to deploy a SA to fill the endpoint directory instead of using
        // data defined in test classes petals-admin-api.

        final Command command = new ListEndpoints();
        command.setShell(this.dummyShellWrapper.getShell());

        command.execute(new String[] { "--" + ListEndpoints.OUTPUT_INTERFACES_LONG_OPTION });

        final String output = this.dummyShellWrapper.getOut().toString();
        assertNotNull("No output", output);
        assertTrue("Empty output", !output.isEmpty());
        assertTrue("Header of section 'Interfaces' displayed", !output.contains("Interfaces:"));
        assertTrue("Interface missing", output.startsWith(EndpointDirectoryViewMock.EP1_ITF1_NAME));
        assertTrue("Endpoint missing as sub element of interface: " + output,
                output.endsWith(EndpointDirectoryViewMock.EP1_NAME + "\n"));
        assertTrue("Output contains a service", !output.contains(EndpointDirectoryViewMock.EP1_SVC1_NAME));
    }

    /**
     * <p>
     * Check the command 'endpoint-list' against the output filtering where:
     * <ul>
     * <li>only endpoints and services are asked to be displayed</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs,</li>
     * <li>the header of the section 'Endpoints' is displayed</li>
     * <li>the endpoint section is displayed</li>
     * <li>the header of the section 'Services' is displayed</li>
     * <li>the service section is displayed</li>
     * <li>the header of the section 'Interfaces' is not displayed</li>
     * </ul>
     * </p>
     */
    @Test
    public void testOutputFilteringLimitedToEndpointsAndServices() throws Exception {
        // TODO: prefer to deploy a SA to fill the endpoint directory instead of using
        // data defined in test classes petals-admin-api.

        final Command command = new ListEndpoints();
        command.setShell(this.dummyShellWrapper.getShell());

        command.execute(new String[] { "--" + ListEndpoints.OUTPUT_ENDPOINTS_LONG_OPTION,
                "--" + ListEndpoints.OUTPUT_SERVICES_LONG_OPTION });

        final String output = this.dummyShellWrapper.getOut().toString();
        assertNotNull("No output", output);
        assertTrue("Empty output", !output.isEmpty());
        assertTrue("Header of section 'Endpoints' not displayed", output.startsWith("Endpoints:"));
        assertTrue("Enpoint missing", output.contains(EndpointDirectoryViewMock.EP1_NAME + ":"));
        assertTrue("Header of section 'Services' not displayed", output.contains("Services:"));
        assertTrue("Service missing", output.contains(EndpointDirectoryViewMock.EP1_SVC1_NAME + ":"));
        assertTrue("Enpoint missing as sub element of service: " + output,
                output.endsWith(EndpointDirectoryViewMock.EP1_NAME + "\n"));
        assertTrue("Header of section 'Interfaces' is displayed", !output.contains("Interfaces:"));
    }

    /**
     * <p>
     * Check the command 'endpoint-list' against the output filtering where:
     * <ul>
     * <li>only endpoints and interfaces are asked to be displayed</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs,</li>
     * <li>the header of the section 'Endpoints' is displayed</li>
     * <li>the endpoint section is displayed</li>
     * <li>the header of the section 'Services' is not displayed</li>
     * <li>the header of the section 'Interfaces' is displayed</li>
     * <li>the interface section is displayed</li>
     * </ul>
     * </p>
     */
    @Test
    public void testOutputFilteringLimitedToEndpointsAndInterfaces() throws Exception {
        // TODO: prefer to deploy a SA to fill the endpoint directory instead of using
        // data defined in test classes petals-admin-api.

        final Command command = new ListEndpoints();
        command.setShell(this.dummyShellWrapper.getShell());

        command.execute(new String[] { "--" + ListEndpoints.OUTPUT_ENDPOINTS_LONG_OPTION,
                "--" + ListEndpoints.OUTPUT_INTERFACES_LONG_OPTION });

        final String output = this.dummyShellWrapper.getOut().toString();
        assertNotNull("No output", output);
        assertTrue("Empty output", !output.isEmpty());
        assertTrue("Header of section 'Endpoints' not displayed", output.startsWith("Endpoints:"));
        assertTrue("Enpoint missing", output.contains(EndpointDirectoryViewMock.EP1_NAME + ":"));
        assertTrue("Header of section 'Services' is displayed", !output.contains("Services:"));
        assertTrue("Header of section 'Interfaces' not displayed", output.contains("Interfaces:"));
        assertTrue("Interface missing", output.contains(EndpointDirectoryViewMock.EP1_ITF1_NAME + ":"));
        assertTrue("Enpoint missing as sub element of interface: " + output,
                output.endsWith(EndpointDirectoryViewMock.EP1_NAME + "\n"));
    }

    /**
     * <p>
     * Check the command 'endpoint-list' against the output filtering where:
     * <ul>
     * <li>only services and interfaces are asked to be displayed</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs,</li>
     * <li>the header of the section 'Endpoints' is not displayed</li>
     * <li>the header of the section 'Services' is displayed</li>
     * <li>the service section is displayed</li>
     * <li>the header of the section 'Interfaces' is displayed</li>
     * <li>the interface section is displayed</li>
     * </ul>
     * </p>
     */
    @Test
    public void testOutputFilteringLimitedToServicesAndInterfaces() throws Exception {
        // TODO: prefer to deploy a SA to fill the endpoint directory instead of using
        // data defined in test classes petals-admin-api.

        final Command command = new ListEndpoints();
        command.setShell(this.dummyShellWrapper.getShell());

        command.execute(new String[] { "--" + ListEndpoints.OUTPUT_SERVICES_LONG_OPTION,
                "--" + ListEndpoints.OUTPUT_INTERFACES_LONG_OPTION });

        final String output = this.dummyShellWrapper.getOut().toString();
        assertNotNull("No output", output);
        assertTrue("Empty output", !output.isEmpty());
        assertTrue("Header of section 'Endpoints' is displayed", !output.contains("Endpoints:"));
        assertTrue("Header of section 'Services' not displayed", output.startsWith("Services:"));
        assertTrue("Service missing", output.contains(EndpointDirectoryViewMock.EP1_SVC1_NAME + ":"));
        assertTrue("Header of section 'Interfaces' not displayed", output.contains("Interfaces:"));
        assertTrue("Interface missing", output.contains(EndpointDirectoryViewMock.EP1_ITF1_NAME + ":"));
        assertTrue("Enpoint missing as sub element of interface: " + output,
                output.endsWith(EndpointDirectoryViewMock.EP1_NAME + "\n"));
    }

    /**
     * <p>
     * Test of the command "endpoint-list" when no connection is established to run the command.
     * </p>
     * <p>
     * Expected results: an error occurs about the connection not established.
     * </p>
     */
    @Test(expected = ConnectionRequiredException.class)
    public void executeNotConnected() throws Exception {

        final StringStreamShell shell = new StringStreamShell();
        final Command command = new ListEndpoints();
        command.setShell(shell);

        command.execute(new String[] {});
    }
}