/**
 * Copyright (c) 2010-2012 EBM WebSourcing, 2012-2017 Linagora
 * 
 * This program/library is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published by
 * the Free Software Foundation, either version 2.1 of the License, or (at your
 * option) any later version.
 * 
 * This program/library is distributed in the hope that it will be useful, but
 * WITHOUT ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or
 * FITNESS FOR A PARTICULAR PURPOSE. See the GNU Lesser General Public License
 * for more details.
 * 
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program/library; If not, see http://www.gnu.org/licenses/
 * for the GNU Lesser General Public License version 2.1.
 */
package org.ow2.petals.cli.shell.command;

import static org.junit.Assert.assertEquals;
import static org.junit.Assert.assertFalse;
import static org.junit.Assert.assertNotNull;
import static org.junit.Assert.assertTrue;
import static org.junit.Assert.fail;
import static org.ow2.petals.cli.Constants.CommonOption.URL_SHORT_OPTION;

import java.net.URL;
import java.util.List;

import org.junit.Test;
import org.ow2.petals.admin.api.ArtifactAdministration;
import org.ow2.petals.admin.api.artifact.Artifact;
import org.ow2.petals.admin.api.artifact.ArtifactState;
import org.ow2.petals.admin.api.artifact.ArtifactUtils;
import org.ow2.petals.admin.api.artifact.Component;
import org.ow2.petals.admin.api.exception.ArtifactNotFoundException;
import org.ow2.petals.admin.api.exception.ArtifactTypeIsNeededException;
import org.ow2.petals.cli.api.command.Command;
import org.ow2.petals.cli.api.command.exception.CommandBadArgumentNumberException;
import org.ow2.petals.cli.api.command.exception.CommandException;
import org.ow2.petals.cli.api.command.exception.CommandMissingArgumentException;
import org.ow2.petals.cli.api.command.exception.CommandTooManyArgumentsException;
import org.ow2.petals.cli.api.command.exception.ConnectionRequiredException;
import org.ow2.petals.cli.base.junit.shell.StringStreamShell;

/**
 * 
 * @author "Mathieu CARROLLE - EBM WebSourcing"
 * 
 */
public class StopArtifactTest extends AbstractArtifactCommandTest {

    /**
     * Test that the command usage is defined
     */
    @Test
    public void testUsage_0() {
        final StopArtifact command = new StopArtifact();
        final String usage = command.getUsage();
        assertNotNull("Command usage null", usage);
        assertFalse("Command usage empty", usage.isEmpty());
        assertTrue("'usage' is missing at the begining of the command usage",
                usage.startsWith("usage"));
        assertTrue("The command name is missing in the command usage",
                usage.contains(command.getName()));
    }

    /**
     * Test of the command "stop-artifact" against argument management, where:
     * <ul>
     * <li>an unregonized option is provided</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about bad argument number.
     * </p>
     */
    @Test
    public void testArgumentsError_0() throws Exception {

        final StopArtifact command = new StopArtifact();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-z" });
            fail("CommandBadArgumentNumberException is not thrown");
        } catch (final CommandBadArgumentNumberException e) {
            assertTrue(
                    "Error label is missing.",
                    e.getMessage()
                            .startsWith(
                                    CommandBadArgumentNumberException.BAD_NUMBER_OF_ARGUMENTS_OR_INCOMPATIBLE_ARGUMENTS));
        }
    }

    @Test
    public void testStopOneArtifactByFileURL() throws Exception {

        final ArtifactAdministration admin = getArtifactAdmin();

        admin.deployAndStartArtifact(COMPONENT_URL, false);

        List<Artifact> artifacts = admin.listArtifacts();
        assertEquals(1, artifacts.size());
        Artifact artifact = admin.getArtifactInfo(COMPONENT_URL);
        assertEquals(ArtifactState.State.STARTED, ((ArtifactState) artifact).getState());
        StopArtifact command = new StopArtifact();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "-" + URL_SHORT_OPTION, COMPONENT_URL.toString() });
        artifact = admin.getArtifactInfo(COMPONENT_URL);
        assertEquals(ArtifactState.State.STOPPED, ((ArtifactState) artifact).getState());
    }

    @Test
    public void testStopOneArtifactByMavenURL() throws Exception {

        final ArtifactAdministration admin = getArtifactAdmin();

        final URL componentURL = new URL("mvn:org.ow2.petals/petals-bc-soap/4.4.2/zip");
        admin.deployAndStartArtifact(componentURL, false);

        final List<Artifact> artifacts = admin.listArtifacts();
        assertEquals(1, artifacts.size());
        Artifact artifact = admin.getArtifactInfo(componentURL);
        assertEquals(ArtifactState.State.STARTED, ((ArtifactState) artifact).getState());
        final StopArtifact command = new StopArtifact();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "-" + URL_SHORT_OPTION, componentURL.toString() });
        artifact = admin.getArtifactInfo(componentURL);
        assertEquals(ArtifactState.State.STOPPED, ((ArtifactState) artifact).getState());
    }

    /**
     * <p>
     * Test of the command "stop-artifact" to stop a started artifact identified
     * by artifact type and artifact identifier where:
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>the artifact type is provided</li>
     * <li>only one JBI artifact is deployed with the identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs</li>
     * <li>artifact is correctly stopped</li>
     * </ul>
     * </p>
     */
    @Test
    public void testStopOneArtifactByTypeAndName_0() throws Exception {

        final ArtifactAdministration admin = getArtifactAdmin();
        admin.deployAndStartArtifact(COMPONENT_URL, false);

        Artifact artifact = admin.getArtifactInfo(COMPONENT_URL);
        String name = artifact.getName();
        String type = artifact.getType();
        List<Artifact> artifacts = admin.listArtifacts();
        assertEquals("Check if component is deployed", 1, artifacts.size());
        assertEquals(ArtifactState.State.STARTED, ((ArtifactState) admin.getArtifactInfo(COMPONENT_URL)).getState());
        StopArtifact command = new StopArtifact();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "-" + ArtifactBasedCommand.ARTIFACT_SHORT_OPTION, name,
                "-" + ArtifactTypeBasedCommand.TYPE_SHORT_OPTION, type });
        artifact = admin.getArtifactInfo(COMPONENT_URL);
        assertEquals(ArtifactState.State.STOPPED, ((ArtifactState) artifact).getState());
    }

    /**
     * <p>
     * Test of the command "stop-artifact" to stop a started artifact identified
     * by artifact type and artifact identifier where:
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>the artifact type is missing</li>
     * <li>only one JBI artifact is deployed with the identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs</li>
     * <li>artifact is correctly stopped</li>
     * </ul>
     * </p>
     */
    @Test
    public void testStopOneArtifactByTypeAndName_1() throws Exception {

        final ArtifactAdministration admin = getArtifactAdmin();
        admin.deployAndStartArtifact(COMPONENT_URL, false);

        Artifact artifact = admin.getArtifactInfo(COMPONENT_URL);
        String name = artifact.getName();
        List<Artifact> artifacts = admin.listArtifacts();
        assertEquals("Check if component is deployed", 1, artifacts.size());
        assertEquals(ArtifactState.State.STARTED, ((ArtifactState) admin.getArtifactInfo(COMPONENT_URL)).getState());
        StopArtifact command = new StopArtifact();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "-" + ArtifactBasedCommand.ARTIFACT_SHORT_OPTION, name });
        artifact = admin.getArtifactInfo(COMPONENT_URL);
        assertEquals(ArtifactState.State.STOPPED, ((ArtifactState) artifact).getState());
    }

    /**
     * <p>
     * Test of the command "stop-artifact" to stop a started artifact identified
     * by its type and its identifier where:
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>the artifact type is missing</li>
     * <li>two JBI artifacts are deployed with the same identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>an error occurs about the missing artifact type</li>
     * <li>the artifacts are not stopped</li>
     * </ul>
     * </p>
     */
    @Test
    public void testStopOneArtifactByTypeAndName_2() throws Exception {

        final ArtifactAdministration admin = getArtifactAdmin();
        admin.deployAndStartArtifact(COMPONENT_URL, false);

        admin.deployAndStartArtifact(SA_SOAP_ID_URL, false);
        Artifact artifact = admin.getArtifactInfo(COMPONENT_URL);
        String name = artifact.getName();
        List<Artifact> artifacts = admin.listArtifacts();
        assertEquals("Check if artifacts are deployed", 10, artifacts.size());
        assertEquals(ArtifactState.State.STARTED, ((ArtifactState) admin.getArtifactInfo(COMPONENT_URL)).getState());
        assertEquals(ArtifactState.State.STARTED, ((ArtifactState) admin.getArtifactInfo(SA_SOAP_ID_URL)).getState());
        StopArtifact command = new StopArtifact();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + ArtifactBasedCommand.ARTIFACT_SHORT_OPTION, name });
            fail("ArtifactTypeIsNeededException is not thrown through CommandException.");
        } catch (final CommandException e) {
            assertNotNull("CommandException cause is missing.", e.getCause());
            assertTrue("CommandException cause is not ArtifactTypeIsNeededException",
                    e.getCause() instanceof ArtifactTypeIsNeededException);
        }
        assertEquals(ArtifactState.State.STARTED, ((ArtifactState) admin.getArtifactInfo(COMPONENT_URL)).getState());
        assertEquals(ArtifactState.State.STARTED, ((ArtifactState) admin.getArtifactInfo(SA_SOAP_ID_URL)).getState());
    }

    /**
     * <p>
     * Test of the command "stop-artifact" to stop a started artifact identified
     * by its type and its identifier where:
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>the artifact type is provided</li>
     * <li>two JBI artifacts are deployed with the same identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>no error occurs</li>
     * <li>the right artifact is stopped</li>
     * <li>the other artifact is not stopped</li>
     * </ul>
     * </p>
     */
    @Test
    public void testStopOneArtifactByTypeAndName_3() throws Exception {

        final ArtifactAdministration admin = getArtifactAdmin();
        admin.deployAndStartArtifact(COMPONENT_URL, false);

        admin.deployAndStartArtifact(SA_SOAP_ID_URL, false);
        Artifact artifact = admin.getArtifactInfo(COMPONENT_URL);
        String name = artifact.getName();
        final String type = artifact.getType();
        List<Artifact> artifacts = admin.listArtifacts();
        assertEquals("Check if artifacts are deployed", 10, artifacts.size());
        assertEquals(ArtifactState.State.STARTED, ((ArtifactState) admin.getArtifactInfo(COMPONENT_URL)).getState());
        assertEquals(ArtifactState.State.STARTED, ((ArtifactState) admin.getArtifactInfo(SA_SOAP_ID_URL)).getState());
        StopArtifact command = new StopArtifact();
        command.setShell(this.dummyShellWrapper.getShell());
        command.execute(new String[] { "-" + ArtifactBasedCommand.ARTIFACT_SHORT_OPTION, name,
                "-" + ArtifactTypeBasedCommand.TYPE_SHORT_OPTION, type });
        assertEquals(ArtifactState.State.STOPPED, ((ArtifactState) admin.getArtifactInfo(COMPONENT_URL)).getState());
        assertEquals(ArtifactState.State.STARTED, ((ArtifactState) admin.getArtifactInfo(SA_SOAP_ID_URL)).getState());
    }

    /**
     * <p>
     * Test of the command "stop-artifact" to stop a started artifact identified by its type and its identifier where:
     * <ul>
     * <li>the artifact identifier is missing,</li>
     * <li>the artifact type is provided,</li>
     * <li>two JBI artifacts are deployed with the same identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>an error occurs about the bas number argument</li>
     * <li>the artifacts are always started</li>
     * </ul>
     * </p>
     */
    @Test(expected = CommandBadArgumentNumberException.class)
    public void testStopOneArtifactByTypeAndName_4() throws Exception {

        final ArtifactAdministration admin = getArtifactAdmin();
        admin.deployAndStartArtifact(COMPONENT_URL, false);

        admin.deployAndStartArtifact(SA_SOAP_ID_URL, false);
        Artifact artifact = admin.getArtifactInfo(COMPONENT_URL);
        final String type = artifact.getType();
        List<Artifact> artifacts = admin.listArtifacts();
        assertEquals("Check if artifacts are deployed", 10, artifacts.size());
        assertEquals(ArtifactState.State.STARTED, ((ArtifactState) admin.getArtifactInfo(COMPONENT_URL)).getState());
        assertEquals(ArtifactState.State.STARTED, ((ArtifactState) admin.getArtifactInfo(SA_SOAP_ID_URL)).getState());
        StopArtifact command = new StopArtifact();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + ArtifactTypeBasedCommand.TYPE_SHORT_OPTION, type });
            fail("ArtifactNotFoundException is not thrown through CommandException.");
        } catch (final CommandBadArgumentNumberException e) {
            // Expected exception
            assertEquals(ArtifactState.State.STARTED,
                    ((ArtifactState) admin.getArtifactInfo(COMPONENT_URL)).getState());
            assertEquals(ArtifactState.State.STARTED,
                    ((ArtifactState) admin.getArtifactInfo(SA_SOAP_ID_URL)).getState());
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "stop-artifact" to stop a started artifact identified
     * by its type and its identifier where:
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>a bad artifact type is provided,</li>
     * <li>two JBI artifacts are deployed with the same identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>an error occurs about artifact not found</li>
     * <li>the artifacts are always started</li>
     * </ul>
     * </p>
     */
    @Test
    public void testStopOneArtifactByTypeAndName_5() throws Exception {

        final ArtifactAdministration admin = getArtifactAdmin();
        admin.deployAndStartArtifact(COMPONENT_URL, false);

        admin.deployAndStartArtifact(SA_SOAP_ID_URL, false);
        Artifact artifact = admin.getArtifactInfo(COMPONENT_URL);
        String name = artifact.getName();
        List<Artifact> artifacts = admin.listArtifacts();
        assertEquals("Check if artifacts are deployed", 10, artifacts.size());
        assertEquals(ArtifactState.State.STARTED, ((ArtifactState) admin.getArtifactInfo(COMPONENT_URL)).getState());
        assertEquals(ArtifactState.State.STARTED, ((ArtifactState) admin.getArtifactInfo(SA_SOAP_ID_URL)).getState());
        StopArtifact command = new StopArtifact();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            assertFalse("The artifact is a SE", "SE".equals(artifact.getType()));
            command.execute(new String[] { "-" + ArtifactBasedCommand.ARTIFACT_SHORT_OPTION, name,
                    "-" + ArtifactTypeBasedCommand.TYPE_SHORT_OPTION, "SE" });
            fail("ArtifactNotFoundException is not thrown through CommandException.");
        } catch (final CommandException e) {
            assertNotNull("CommandException cause is missing.", e.getCause());
            assertTrue("CommandException cause is not ArtifactNotFoundException",
                    e.getCause() instanceof ArtifactNotFoundException);
        }
        assertEquals(ArtifactState.State.STARTED, ((ArtifactState) admin.getArtifactInfo(COMPONENT_URL)).getState());
        assertEquals(ArtifactState.State.STARTED, ((ArtifactState) admin.getArtifactInfo(SA_SOAP_ID_URL)).getState());
    }

    /**
     * <p>
     * Test of the command "stop-artifact" to stop a started artifact identified
     * by artifact type and artifact identifier where:
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>a bad artifact type is provided,</li>
     * <li>only one JBI artifact is deployed with the identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>an error occurs about artifact not found</li>
     * <li>the artifact is always started</li>
     * </ul>
     * </p>
     */
    @Test
    public void testStopOneArtifactByTypeAndName_6() throws Exception {

        final ArtifactAdministration admin = getArtifactAdmin();
        admin.deployAndStartArtifact(COMPONENT_URL, false);

        Artifact artifact = admin.getArtifactInfo(COMPONENT_URL);
        String name = artifact.getName();
        List<Artifact> artifacts = admin.listArtifacts();
        assertEquals("Check if component is deployed", 1, artifacts.size());
        assertEquals(ArtifactState.State.STARTED, ((ArtifactState) admin.getArtifactInfo(COMPONENT_URL)).getState());
        StopArtifact command = new StopArtifact();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            assertFalse("The artifact is a SE", "SE".equals(artifact.getType()));
            command.execute(new String[] { "-" + ArtifactBasedCommand.ARTIFACT_SHORT_OPTION, name,
                    "-" + ArtifactTypeBasedCommand.TYPE_SHORT_OPTION, "SE" });
            fail("ArtifactTypeIsNeededException is not thrown through CommandException.");
        } catch (final CommandException e) {
            assertNotNull("CommandException cause is missing.", e.getCause());
            assertTrue("CommandException cause is not ArtifactNotFoundException",
                    e.getCause() instanceof ArtifactNotFoundException);
        }
        assertEquals(ArtifactState.State.STARTED, ((ArtifactState) admin.getArtifactInfo(COMPONENT_URL)).getState());
    }

    /**
     * <p>
     * Test of the command "stop-artifact" to stop a started artifact identified by artifact type and artifact
     * identifier where:
     * <ul>
     * <li>the artifact identifier is missing,</li>
     * <li>the artifact type is provided,</li>
     * <li>only one JBI artifact is deployed with the identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results:
     * <ul>
     * <li>an error occurs about the bas number argument</li>
     * <li>the artifact is always started</li>
     * </ul>
     * </p>
     */
    @Test(expected = CommandBadArgumentNumberException.class)
    public void testStopOneArtifactByTypeAndName_7() throws Exception {

        final ArtifactAdministration admin = getArtifactAdmin();
        admin.deployAndStartArtifact(COMPONENT_URL, false);

        Artifact artifact = admin.getArtifactInfo(COMPONENT_URL);
        String type = artifact.getType();
        List<Artifact> artifacts = admin.listArtifacts();
        assertEquals("Check if component is deployed", 1, artifacts.size());
        assertEquals(ArtifactState.State.STARTED, ((ArtifactState) admin.getArtifactInfo(COMPONENT_URL)).getState());
        StopArtifact command = new StopArtifact();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + ArtifactTypeBasedCommand.TYPE_SHORT_OPTION, type });
            fail("ArtifactNotFoundException is not thrown through CommandException.");
        } catch (final CommandBadArgumentNumberException e) {
            // Expected exception
            assertEquals(ArtifactState.State.STARTED,
                    ((ArtifactState) admin.getArtifactInfo(COMPONENT_URL)).getState());
            throw e;
        }
    }

    /**
     * <p>
     * Test of the command "stop-artifact" to stop a started artifact identified by artifact type and artifact
     * identifier where:
     * <ul>
     * <li>the artifact identifier is provided,</li>
     * <li>a artifact type option is provided without value,</li>
     * <li>only one JBI artifact is deployed with the identifier.</li>
     * </ul>
     * </p>
     * <p>
     * Expected results: an error occurs about artifact not found
     * </p>
     */
    @Test
    public void testStopOneArtifactByTypeAndName_8() throws Exception {
        this.deployStartArtifact(COMPONENT_URL);

        Component a = (Component) ArtifactUtils.createArtifact(COMPONENT_URL);

        StopArtifact command = new StopArtifact();
        command.setShell(this.dummyShellWrapper.getShell());
        String identifier = a.getName();
        try {
            assertFalse("The artifact is a SE", "SE".equals(a.getType()));
            command.execute(new String[] { "-" + ArtifactBasedCommand.ARTIFACT_SHORT_OPTION, identifier,
                    "-" + ArtifactTypeBasedCommand.TYPE_SHORT_OPTION });
            fail("CommandMissingArgumentException is not thrown");
        } catch (final CommandMissingArgumentException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandMissingArgumentException.MISSING_ARGUMENT));
            assertTrue("The option '-" + StopArtifact.TYPE_SHORT_OPTION + "' has a missing argument.",
                    StopArtifact.TYPE_SHORT_OPTION.equals(e.getOption().getOpt()));
            assertTrue("Missing argument is missing in error message.",
                    e.getMessage().endsWith(StopArtifact.TYPE_SHORT_OPTION));
        }
    }

    @Test
    public void testStopManyArtifactThrownException() throws Exception {

        StopArtifact command = new StopArtifact();
        command.setShell(this.dummyShellWrapper.getShell());
        try {
            command.execute(new String[] { "-" + URL_SHORT_OPTION, COMPONENT_URL.toString(),
                    SA_URL.toString() });

            fail("Exception 'CommandTooManyArgumentsException' is not thrown");
        } catch (final CommandTooManyArgumentsException e) {
            assertTrue("Error label is missing.",
                    e.getMessage().startsWith(CommandTooManyArgumentsException.TOO_MANY_ARGUMENTS));
        }
    }

    /**
     * <p>
     * Test of the command "stop-artifact" when no connection is established to run the command.
     * </p>
     * <p>
     * Expected results: an error occurs about the connection not established.
     * </p>
     */
    @Test(expected = ConnectionRequiredException.class)
    public void executeNotConnected() throws CommandException {

        final StringStreamShell shell = new StringStreamShell();
        final Command command = new StopArtifact();
        command.setShell(shell);

        command.execute(new String[] {});
    }
}